from typing import Literal

# HTTP Status Codes

HTTP_100_CONTINUE: Literal[100] = 100
"""HTTP status code 'Continue'"""

HTTP_101_SWITCHING_PROTOCOLS: Literal[101] = 101
"""HTTP status code 'Switching Protocols'"""

HTTP_102_PROCESSING: Literal[102] = 102
"""HTTP status code 'Processing'"""

HTTP_103_EARLY_HINTS: Literal[103] = 103
"""HTTP status code 'Early Hints'"""

HTTP_200_OK: Literal[200] = 200
"""HTTP status code 'OK'"""

HTTP_201_CREATED: Literal[201] = 201
"""HTTP status code 'Created'"""

HTTP_202_ACCEPTED: Literal[202] = 202
"""HTTP status code 'Accepted'"""

HTTP_203_NON_AUTHORITATIVE_INFORMATION: Literal[203] = 203
"""HTTP status code 'Non Authoritative Information'"""

HTTP_204_NO_CONTENT: Literal[204] = 204
"""HTTP status code 'No Content'"""

HTTP_205_RESET_CONTENT: Literal[205] = 205
"""HTTP status code 'Reset Content'"""

HTTP_206_PARTIAL_CONTENT: Literal[206] = 206
"""HTTP status code 'Partial Content'"""

HTTP_207_MULTI_STATUS: Literal[207] = 207
"""HTTP status code 'Multi Status'"""

HTTP_208_ALREADY_REPORTED: Literal[208] = 208
"""HTTP status code 'Already Reported'"""

HTTP_226_IM_USED: Literal[226] = 226
"""HTTP status code 'I'm Used'"""

HTTP_300_MULTIPLE_CHOICES: Literal[300] = 300
"""HTTP status code 'Multiple Choices'"""

HTTP_301_MOVED_PERMANENTLY: Literal[301] = 301
"""HTTP status code 'Moved Permanently'"""

HTTP_302_FOUND: Literal[302] = 302
"""HTTP status code 'Found'"""

HTTP_303_SEE_OTHER: Literal[303] = 303
"""HTTP status code 'See Other'"""

HTTP_304_NOT_MODIFIED: Literal[304] = 304
"""HTTP status code 'Not Modified'"""

HTTP_305_USE_PROXY: Literal[305] = 305
"""HTTP status code 'Use Proxy'"""

HTTP_306_RESERVED: Literal[306] = 306
"""HTTP status code 'Reserved'"""

HTTP_307_TEMPORARY_REDIRECT: Literal[307] = 307
"""HTTP status code 'Temporary Redirect'"""

HTTP_308_PERMANENT_REDIRECT: Literal[308] = 308
"""HTTP status code 'Permanent Redirect'"""

HTTP_400_BAD_REQUEST: Literal[400] = 400
"""HTTP status code 'Bad Request'"""

HTTP_401_UNAUTHORIZED: Literal[401] = 401
"""HTTP status code 'Unauthorized'"""

HTTP_402_PAYMENT_REQUIRED: Literal[402] = 402
"""HTTP status code 'Payment Required'"""

HTTP_403_FORBIDDEN: Literal[403] = 403
"""HTTP status code 'Forbidden'"""

HTTP_404_NOT_FOUND: Literal[404] = 404
"""HTTP status code 'Not Found'"""

HTTP_405_METHOD_NOT_ALLOWED: Literal[405] = 405
"""HTTP status code 'Method Not Allowed'"""

HTTP_406_NOT_ACCEPTABLE: Literal[406] = 406
"""HTTP status code 'Not Acceptable'"""

HTTP_407_PROXY_AUTHENTICATION_REQUIRED: Literal[407] = 407
"""HTTP status code 'Proxy Authentication Required'"""

HTTP_408_REQUEST_TIMEOUT: Literal[408] = 408
"""HTTP status code 'Request Timeout'"""

HTTP_409_CONFLICT: Literal[409] = 409
"""HTTP status code 'Conflict'"""

HTTP_410_GONE: Literal[410] = 410
"""HTTP status code 'Gone'"""

HTTP_411_LENGTH_REQUIRED: Literal[411] = 411
"""HTTP status code 'Length Required'"""

HTTP_412_PRECONDITION_FAILED: Literal[412] = 412
"""HTTP status code 'Precondition Failed'"""

HTTP_413_REQUEST_ENTITY_TOO_LARGE: Literal[413] = 413
"""HTTP status code 'Request Entity Too Large'"""

HTTP_414_REQUEST_URI_TOO_LONG: Literal[414] = 414
"""HTTP status code 'Request URI Too Long'"""

HTTP_415_UNSUPPORTED_MEDIA_TYPE: Literal[415] = 415
"""HTTP status code 'Unsupported Media Type'"""

HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE: Literal[416] = 416
"""HTTP status code 'Requested Range Not Satisfiable'"""

HTTP_417_EXPECTATION_FAILED: Literal[417] = 417
"""HTTP status code 'Expectation Failed'"""

HTTP_418_IM_A_TEAPOT: Literal[418] = 418
"""HTTP status code 'I'm A Teapot'"""

HTTP_421_MISDIRECTED_REQUEST: Literal[421] = 421
"""HTTP status code 'Misdirected Request'"""

HTTP_422_UNPROCESSABLE_ENTITY: Literal[422] = 422
"""HTTP status code 'Unprocessable Entity'"""

HTTP_423_LOCKED: Literal[423] = 423
"""HTTP status code 'Locked'"""

HTTP_424_FAILED_DEPENDENCY: Literal[424] = 424
"""HTTP status code 'Failed Dependency'"""

HTTP_425_TOO_EARLY: Literal[425] = 425
"""HTTP status code 'Too Early'"""

HTTP_426_UPGRADE_REQUIRED: Literal[426] = 426
"""HTTP status code 'Upgrade Required'"""

HTTP_428_PRECONDITION_REQUIRED: Literal[428] = 428
"""HTTP status code 'Precondition Required'"""

HTTP_429_TOO_MANY_REQUESTS: Literal[429] = 429
"""HTTP status code 'Too Many Requests'"""

HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE: Literal[431] = 431
"""HTTP status code 'Request Header Fields Too Large'"""

HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS: Literal[451] = 451
"""HTTP status code 'Unavailable For Legal Reasons'"""

HTTP_500_INTERNAL_SERVER_ERROR: Literal[500] = 500
"""HTTP status code 'Internal Server Error'"""

HTTP_501_NOT_IMPLEMENTED: Literal[501] = 501
"""HTTP status code 'Not Implemented'"""

HTTP_502_BAD_GATEWAY: Literal[502] = 502
"""HTTP status code 'Bad Gateway'"""

HTTP_503_SERVICE_UNAVAILABLE: Literal[503] = 503
"""HTTP status code 'Service Unavailable'"""

HTTP_504_GATEWAY_TIMEOUT: Literal[504] = 504
"""HTTP status code 'Gateway Timeout'"""

HTTP_505_HTTP_VERSION_NOT_SUPPORTED: Literal[505] = 505
"""HTTP status code 'Http Version Not Supported'"""

HTTP_506_VARIANT_ALSO_NEGOTIATES: Literal[506] = 506
"""HTTP status code 'Variant Also Negotiates'"""

HTTP_507_INSUFFICIENT_STORAGE: Literal[507] = 507
"""HTTP status code 'Insufficient Storage'"""

HTTP_508_LOOP_DETECTED: Literal[508] = 508
"""HTTP status code 'Loop Detected'"""

HTTP_510_NOT_EXTENDED: Literal[510] = 510
"""HTTP status code 'Not Extended'"""

HTTP_511_NETWORK_AUTHENTICATION_REQUIRED: Literal[511] = 511
"""HTTP status code 'Network Authentication Required'"""


# Websocket Codes
WS_1000_NORMAL_CLOSURE: Literal[1000] = 1000
"""WebSocket status code 'Normal Closure'"""

WS_1001_GOING_AWAY: Literal[1001] = 1001
"""WebSocket status code 'Going Away'"""

WS_1002_PROTOCOL_ERROR: Literal[1002] = 1002
"""WebSocket status code 'Protocol Error'"""

WS_1003_UNSUPPORTED_DATA: Literal[1003] = 1003
"""WebSocket status code 'Unsupported Data'"""

WS_1005_NO_STATUS_RECEIVED: Literal[1005] = 1005
"""WebSocket status code 'No Status Received'"""

WS_1006_ABNORMAL_CLOSURE: Literal[1006] = 1006
"""WebSocket status code 'Abnormal Closure'"""

WS_1007_INVALID_FRAME_PAYLOAD_DATA: Literal[1007] = 1007
"""WebSocket status code 'Invalid Frame Payload Data'"""

WS_1008_POLICY_VIOLATION: Literal[1008] = 1008
"""WebSocket status code 'Policy Violation'"""

WS_1009_MESSAGE_TOO_BIG: Literal[1009] = 1009
"""WebSocket status code 'Message Too Big'"""

WS_1010_MANDATORY_EXT: Literal[1010] = 1010
"""WebSocket status code 'Mandatory Ext.'"""

WS_1011_INTERNAL_ERROR: Literal[1011] = 1011
"""WebSocket status code 'Internal Error'"""

WS_1012_SERVICE_RESTART: Literal[1012] = 1012
"""WebSocket status code 'Service Restart'"""

WS_1013_TRY_AGAIN_LATER: Literal[1013] = 1013
"""WebSocket status code 'Try Again Later'"""

WS_1014_BAD_GATEWAY: Literal[1014] = 1014
"""WebSocket status code 'Bad Gateway'"""

WS_1015_TLS_HANDSHAKE: Literal[1015] = 1015
"""WebSocket status code 'TLS Handshake'"""


__all__ = (
    "HTTP_100_CONTINUE",
    "HTTP_101_SWITCHING_PROTOCOLS",
    "HTTP_102_PROCESSING",
    "HTTP_103_EARLY_HINTS",
    "HTTP_200_OK",
    "HTTP_201_CREATED",
    "HTTP_202_ACCEPTED",
    "HTTP_203_NON_AUTHORITATIVE_INFORMATION",
    "HTTP_204_NO_CONTENT",
    "HTTP_205_RESET_CONTENT",
    "HTTP_206_PARTIAL_CONTENT",
    "HTTP_207_MULTI_STATUS",
    "HTTP_208_ALREADY_REPORTED",
    "HTTP_226_IM_USED",
    "HTTP_300_MULTIPLE_CHOICES",
    "HTTP_301_MOVED_PERMANENTLY",
    "HTTP_302_FOUND",
    "HTTP_303_SEE_OTHER",
    "HTTP_304_NOT_MODIFIED",
    "HTTP_305_USE_PROXY",
    "HTTP_306_RESERVED",
    "HTTP_307_TEMPORARY_REDIRECT",
    "HTTP_308_PERMANENT_REDIRECT",
    "HTTP_400_BAD_REQUEST",
    "HTTP_401_UNAUTHORIZED",
    "HTTP_402_PAYMENT_REQUIRED",
    "HTTP_403_FORBIDDEN",
    "HTTP_404_NOT_FOUND",
    "HTTP_405_METHOD_NOT_ALLOWED",
    "HTTP_406_NOT_ACCEPTABLE",
    "HTTP_407_PROXY_AUTHENTICATION_REQUIRED",
    "HTTP_408_REQUEST_TIMEOUT",
    "HTTP_409_CONFLICT",
    "HTTP_410_GONE",
    "HTTP_411_LENGTH_REQUIRED",
    "HTTP_412_PRECONDITION_FAILED",
    "HTTP_413_REQUEST_ENTITY_TOO_LARGE",
    "HTTP_414_REQUEST_URI_TOO_LONG",
    "HTTP_415_UNSUPPORTED_MEDIA_TYPE",
    "HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE",
    "HTTP_417_EXPECTATION_FAILED",
    "HTTP_418_IM_A_TEAPOT",
    "HTTP_421_MISDIRECTED_REQUEST",
    "HTTP_422_UNPROCESSABLE_ENTITY",
    "HTTP_423_LOCKED",
    "HTTP_424_FAILED_DEPENDENCY",
    "HTTP_425_TOO_EARLY",
    "HTTP_426_UPGRADE_REQUIRED",
    "HTTP_428_PRECONDITION_REQUIRED",
    "HTTP_429_TOO_MANY_REQUESTS",
    "HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE",
    "HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS",
    "HTTP_500_INTERNAL_SERVER_ERROR",
    "HTTP_501_NOT_IMPLEMENTED",
    "HTTP_502_BAD_GATEWAY",
    "HTTP_503_SERVICE_UNAVAILABLE",
    "HTTP_504_GATEWAY_TIMEOUT",
    "HTTP_505_HTTP_VERSION_NOT_SUPPORTED",
    "HTTP_506_VARIANT_ALSO_NEGOTIATES",
    "HTTP_507_INSUFFICIENT_STORAGE",
    "HTTP_508_LOOP_DETECTED",
    "HTTP_510_NOT_EXTENDED",
    "HTTP_511_NETWORK_AUTHENTICATION_REQUIRED",
    "WS_1000_NORMAL_CLOSURE",
    "WS_1001_GOING_AWAY",
    "WS_1002_PROTOCOL_ERROR",
    "WS_1003_UNSUPPORTED_DATA",
    "WS_1005_NO_STATUS_RECEIVED",
    "WS_1006_ABNORMAL_CLOSURE",
    "WS_1007_INVALID_FRAME_PAYLOAD_DATA",
    "WS_1008_POLICY_VIOLATION",
    "WS_1009_MESSAGE_TOO_BIG",
    "WS_1010_MANDATORY_EXT",
    "WS_1011_INTERNAL_ERROR",
    "WS_1012_SERVICE_RESTART",
    "WS_1013_TRY_AGAIN_LATER",
    "WS_1014_BAD_GATEWAY",
    "WS_1015_TLS_HANDSHAKE",
)



from litestar.utils.deprecation import deprecated, warn_deprecation

from .helpers import Ref, get_enum_string_value, get_name
from .path import join_paths, normalize_path
from .predicates import (
    is_any,
    is_attrs_class,
    is_class_and_subclass,
    is_class_var,
    is_dataclass_class,
    is_generic,
    is_mapping,
    is_non_string_iterable,
    is_non_string_sequence,
    is_optional_union,
    is_pydantic_constrained_field,
    is_pydantic_model_class,
    is_pydantic_model_instance,
    is_typed_dict,
    is_union,
)
from .scope import (
    delete_litestar_scope_state,
    get_litestar_scope_state,
    get_serializer_from_scope,
    set_litestar_scope_state,
)
from .sequence import compact, find_index, unique
from .sync import (
    AsyncCallable,
    AsyncIteratorWrapper,
    as_async_callable_list,
    async_partial,
    is_async_callable,
)
from .typing import annotation_is_iterable_of_type, get_origin_or_inner_type, make_non_optional_union

__all__ = (
    "AsyncCallable",
    "AsyncIteratorWrapper",
    "Ref",
    "annotation_is_iterable_of_type",
    "as_async_callable_list",
    "async_partial",
    "compact",
    "delete_litestar_scope_state",
    "deprecated",
    "find_index",
    "get_enum_string_value",
    "get_litestar_scope_state",
    "get_name",
    "get_origin_or_inner_type",
    "get_serializer_from_scope",
    "is_any",
    "is_async_callable",
    "is_attrs_class",
    "is_class_and_subclass",
    "is_class_var",
    "is_dataclass_class",
    "is_generic",
    "is_mapping",
    "is_non_string_iterable",
    "is_non_string_sequence",
    "is_optional_union",
    "is_pydantic_constrained_field",
    "is_pydantic_model_class",
    "is_pydantic_model_instance",
    "is_typed_dict",
    "is_union",
    "join_paths",
    "make_non_optional_union",
    "normalize_path",
    "set_litestar_scope_state",
    "unique",
    "warn_deprecation",
)



from __future__ import annotations

from inspect import getmembers, isclass
from typing import TYPE_CHECKING, Any, Literal, cast

from litestar.constants import SKIP_VALIDATION_NAMES
from litestar.exceptions import ImproperlyConfiguredException
from litestar.params import DependencyKwarg
from litestar.types import AnyCallable, Empty
from litestar.utils.helpers import unwrap_partial
from litestar.utils.predicates import is_attrs_class

try:
    import pydantic

    from litestar._signature.models.pydantic_signature_model import PydanticSignatureModel

    pydantic_types: tuple[Any, ...] = tuple(
        cls for _, cls in getmembers(pydantic.types, isclass) if "pydantic.types" in repr(cls)
    )
except ImportError:  # pragma: no cover
    PydanticSignatureModel = Empty  # type: ignore
    pydantic_types = ()

try:
    from litestar._signature.models.attrs_signature_model import AttrsSignatureModel
except ImportError:
    AttrsSignatureModel = Empty  # type: ignore

if TYPE_CHECKING:
    from typing_extensions import TypeAlias

    from litestar._signature.models.base import SignatureModel
    from litestar.utils.signature import ParsedParameter, ParsedSignature

__all__ = (
    "create_signature_model",
    "get_signature_model",
)


SignatureModelType: TypeAlias = "type[SignatureModel]"


def create_signature_model(
    dependency_name_set: set[str],
    fn: AnyCallable,
    preferred_validation_backend: Literal["pydantic", "attrs"],
    parsed_signature: ParsedSignature,
) -> type[SignatureModel]:
    """Create a model for a callable's signature. The model can than be used to parse and validate before passing it to
    the callable.

    Args:
        dependency_name_set: A set of dependency names
        fn: A callable.
        preferred_validation_backend: Validation/Parsing backend to prefer, if installed
        parsed_signature: A parsed signature for the handler/dependency function.

    Returns:
        A signature model.
    """

    unwrapped_fn = cast("AnyCallable", unwrap_partial(fn))
    fn_name = getattr(fn, "__name__", "anonymous")
    fn_module = getattr(fn, "__module__", None)

    if fn_name == "<lambda>":
        fn_name = "anonymous"

    dependency_names = _validate_dependencies(
        dependency_name_set=dependency_name_set, fn=unwrapped_fn, parsed_signature=parsed_signature
    )

    model_class = _get_signature_model_type(
        preferred_validation_backend=preferred_validation_backend, parsed_signature=parsed_signature
    )

    type_overrides = _create_type_overrides(parsed_signature)

    return model_class.create(
        fn_name=fn_name,
        fn_module=fn_module,
        parsed_signature=parsed_signature,
        dependency_names={*dependency_name_set, *dependency_names},
        type_overrides=type_overrides,
    )


def get_signature_model(value: Any) -> type[SignatureModel]:
    """Retrieve and validate the signature model from a provider or handler."""
    try:
        return cast(SignatureModelType, value.signature_model)
    except AttributeError as e:  # pragma: no cover
        raise ImproperlyConfiguredException(f"The 'signature_model' attribute for {value} is not set") from e


def _any_attrs_annotation(parsed_signature: ParsedSignature) -> bool:
    for parameter in parsed_signature.parameters.values():
        parsed_type = parameter.parsed_type
        if any(is_attrs_class(t.annotation) for t in parsed_type.inner_types) or is_attrs_class(parsed_type.annotation):
            return True
    return False


def _any_pydantic_annotation(parsed_signature: ParsedSignature) -> bool:
    for parameter in parsed_signature.parameters.values():
        parsed_type = parameter.parsed_type
        if any(_is_pydantic_annotation(t.annotation) for t in parsed_type.inner_types) or _is_pydantic_annotation(
            parsed_type.annotation
        ):
            return True
    return False


def _create_type_overrides(parsed_signature: ParsedSignature) -> dict[str, Any]:
    type_overrides = {}
    for parameter in parsed_signature.parameters.values():
        if _should_skip_validation(parameter):
            type_overrides[parameter.name] = Any
    return type_overrides


def _get_signature_model_type(
    preferred_validation_backend: Literal["pydantic", "attrs"],
    parsed_signature: ParsedSignature,
) -> type[SignatureModel]:
    pydantic_installed = PydanticSignatureModel is not Empty  # type: ignore[comparison-overlap]
    attrs_installed = AttrsSignatureModel is not Empty  # type: ignore[comparison-overlap]
    if (
        pydantic_installed
        and (not attrs_installed or not _any_attrs_annotation(parsed_signature))
        and (preferred_validation_backend == "pydantic" or _any_pydantic_annotation(parsed_signature))
    ):
        return cast(SignatureModelType, PydanticSignatureModel)
    return cast(SignatureModelType, AttrsSignatureModel)


def _is_pydantic_annotation(annotation: Any) -> bool:
    return annotation in pydantic_types or hasattr(annotation, "__get_validators__")


def _should_skip_validation(parameter: ParsedParameter) -> bool:
    """Whether the parameter should skip validation.

    Returns:
        A boolean indicating whether the parameter should be validated.
    """
    return parameter.name in SKIP_VALIDATION_NAMES or (
        isinstance(parameter.default, DependencyKwarg) and parameter.default.skip_validation
    )


def _validate_dependencies(
    dependency_name_set: set[str], fn: AnyCallable, parsed_signature: ParsedSignature
) -> set[str]:
    """Validate dependencies of ``parsed_signature``.

    Args:
        dependency_name_set: A set of dependency names
        fn: A callable.
        parsed_signature: A parsed signature.

    Returns:
        A set of validated dependency names.
    """
    fn_name = getattr(fn, "__name__", "anonymous")

    dependency_names: set[str] = set()

    for parameter in parsed_signature.parameters.values():
        if isinstance(parameter.default, DependencyKwarg) and parameter.name not in dependency_name_set:
            if not parameter.parsed_type.is_optional and (
                isinstance(parameter.default, DependencyKwarg) and parameter.default.default is Empty
            ):
                raise ImproperlyConfiguredException(
                    f"Explicit dependency '{parameter.name}' for '{fn_name}' has no default value, "
                    f"or provided dependency."
                )
            dependency_names.add(parameter.name)
    return dependency_names



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING, Any

from litestar.exceptions import ImproperlyConfiguredException

if TYPE_CHECKING:
    from litestar.openapi.spec import Example

__all__ = ("ResponseHeader",)


@dataclass
class ResponseHeader:
    """Container type for a response header."""

    name: str
    """Header name"""

    documentation_only: bool = False
    """Defines the ResponseHeader instance as for OpenAPI documentation purpose only."""

    value: str | None = None
    """Value to set for the response header."""

    description: str | None = None
    """A brief description of the parameter. This could contain examples of
    use.

    [CommonMark syntax](https://spec.commonmark.org/) MAY be used for
    rich text representation.
    """

    required: bool = False
    """Determines whether this parameter is mandatory.

    If the [parameter location](https://spec.openapis.org/oas/v3.1.0#parameterIn) is `"path"`, this property is **REQUIRED** and its value MUST be `true`.
    Otherwise, the property MAY be included and its default value is `false`.
    """

    deprecated: bool = False
    """Specifies that a parameter is deprecated and SHOULD be transitioned out
    of usage.

    Default value is `false`.
    """

    allow_empty_value: bool = False
    """Sets the ability to pass empty-valued parameters. This is valid only for
    `query` parameters and allows sending a parameter with an empty value.
    Default value is `false`. If.

    [style](https://spec.openapis.org/oas/v3.1.0#parameterStyle) is used, and if behavior is `n/a` (cannot be
    serialized), the value of `allowEmptyValue` SHALL be ignored. Use of this property is NOT RECOMMENDED, as it is
    likely to be removed in a later revision.

    The rules for serialization of the parameter are specified in one of two ways.
    For simpler scenarios, a [schema](https://spec.openapis.org/oas/v3.1.0#parameterSchema) and [style](https://spec.openapis.org/oas/v3.1.0#parameterStyle)
    can describe the structure and syntax of the parameter.
    """

    style: str | None = None
    """Describes how the parameter value will be serialized depending on the
    type of the parameter value. Default values (based on value of `in`):

    - for `query` - `form`;
    - for `path` - `simple`;
    - for `header` - `simple`;
    - for `cookie` - `form`.
    """

    explode: bool | None = None
    """When this is true, parameter values of type `array` or `object` generate
    separate parameters for each value of the array or key-value pair of the
    map.

    For other types of parameters this property has no effect.
    When [style](https://spec.openapis.org/oas/v3.1.0#parameterStyle) is `form`, the default value is `true`.
    For all other styles, the default value is `false`.
    """

    allow_reserved: bool = False
    """Determines whether the parameter value SHOULD allow reserved characters,
    as defined by.

    [RFC3986](https://tools.ietf.org/html/rfc3986#section-2.2) `:/?#[]@!$&'()*+,;=` to be included without percent-
    encoding.

    This property only applies to parameters with an `in` value of `query`. The default value is `false`.
    """

    example: Any | None = None
    """Example of the parameter's potential value.

    The example SHOULD match the specified schema and encoding
    properties if present. The `example` field is mutually exclusive of
    the `examples` field. Furthermore, if referencing a `schema` that
    contains an example, the `example` value SHALL _override_ the
    example provided by the schema. To represent examples of media types
    that cannot naturally be represented in JSON or YAML, a string value
    can contain the example with escaping where necessary.
    """

    examples: dict[str, Example] | None = None
    """Examples of the parameter's potential value. Each example SHOULD contain
    a value in the correct format as specified in the parameter encoding. The
    `examples` field is mutually exclusive of the `example` field. Furthermore,
    if referencing a `schema` that contains an example, the `examples` value
    SHALL _override_ the example provided by the schema.

    For more complex scenarios, the [content](https://spec.openapis.org/oas/v3.1.0#parameterContent) property
    can define the media type and schema of the parameter.
    A parameter MUST contain either a `schema` property, or a `content` property, but not both.
    When `example` or `examples` are provided in conjunction with the `schema` object,
    the example MUST follow the prescribed serialization strategy for the parameter.
    """

    def __post_init__(self) -> None:
        """Ensure that either value is set or the instance is for documentation_only."""
        if not self.documentation_only and self.value is None:
            raise ImproperlyConfiguredException("value must be set if documentation_only is false")

    def __hash__(self) -> int:
        return hash(self.name)



from __future__ import annotations

from litestar.exceptions import ImproperlyConfiguredException

__all__ = ["DTOException", "UnsupportedType"]


class DTOException(ImproperlyConfiguredException):
    """Base exception for DTO errors."""


class UnsupportedType(DTOException):
    """Raised when a type is not supported by Litestar."""



from .asgi_handlers import ASGIRouteHandler, asgi
from .base import BaseRouteHandler
from .http_handlers import HTTPRouteHandler, delete, get, head, patch, post, put, route
from .websocket_handlers import WebsocketListener, WebsocketRouteHandler, websocket, websocket_listener

__all__ = (
    "ASGIRouteHandler",
    "BaseRouteHandler",
    "HTTPRouteHandler",
    "WebsocketListener",
    "WebsocketRouteHandler",
    "asgi",
    "delete",
    "get",
    "head",
    "patch",
    "post",
    "put",
    "route",
    "websocket",
    "websocket_listener",
)



from __future__ import annotations

from typing import TYPE_CHECKING, Callable

if TYPE_CHECKING:
    from .base import Store


from .memory import MemoryStore

__all__ = ("StoreRegistry",)


def default_default_factory(name: str) -> Store:
    return MemoryStore()


class StoreRegistry:
    """Registry for :class:`Store <.base.Store>` instances."""

    __slots__ = ("_stores", "_default_factory")

    def __init__(
        self, stores: dict[str, Store] | None = None, default_factory: Callable[[str], Store] = default_default_factory
    ) -> None:
        """Initialize ``StoreRegistry``.

        Args:
            stores: A dictionary mapping store names to stores, used to initialize the registry
            default_factory: A callable used by :meth:`StoreRegistry.get` to provide a store, if the requested name hasn't
                been registered yet. This callable receives the requested name and should return a
                :class:`Store <.base.Store>` instance.
        """
        self._stores = stores or {}
        self._default_factory = default_factory

    def register(self, name: str, store: Store, allow_override: bool = False) -> None:
        """Register a new :class:`Store <.base.Store>`.

        Args:
            name: Name to register the store under
            store: The store to register
            allow_override: Whether to allow overriding an existing store of the same name

        Raises:
            ValueError: If a store is already registered under this name and ``override`` is not ``True``
        """
        if not allow_override and name in self._stores:
            raise ValueError(f"Store with the name {name!r} already exists")
        self._stores[name] = store

    def get(self, name: str) -> Store:
        """Get a store registered under ``name``. If no such store is registered, create a store using the default
        factory with ``name`` and register the returned store under ``name``.

        Args:
            name: Name of the store

        Returns:
            A :class:`Store <.base.Store>`
        """
        store = self._stores.get(name)
        if not store:
            store = self._stores[name] = self._default_factory(name)
        return store



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.exceptions import MissingDependencyException
from litestar.middleware.base import AbstractMiddleware

__all__ = ("OpenTelemetryInstrumentationMiddleware",)


try:
    import opentelemetry  # noqa: F401
except ImportError as e:
    raise MissingDependencyException("opentelemetry") from e

from opentelemetry.instrumentation.asgi import OpenTelemetryMiddleware
from opentelemetry.util.http import get_excluded_urls

if TYPE_CHECKING:
    from litestar.contrib.opentelemetry import OpenTelemetryConfig
    from litestar.types import ASGIApp, Receive, Scope, Send


class OpenTelemetryInstrumentationMiddleware(AbstractMiddleware):
    """OpenTelemetry Middleware."""

    __slots__ = ("open_telemetry_middleware",)

    def __init__(self, app: ASGIApp, config: OpenTelemetryConfig):
        """Middleware that adds OpenTelemetry instrumentation to the application.

        Args:
            app: The ``next`` ASGI app to call.
            config: An instance of :class:`OpenTelemetryConfig <.contrib.opentelemetry.OpenTelemetryConfig>`
        """
        super().__init__(app=app, scopes=config.scopes, exclude=config.exclude, exclude_opt_key=config.exclude_opt_key)
        self.open_telemetry_middleware = OpenTelemetryMiddleware(
            app=app,
            client_request_hook=config.client_request_hook_handler,
            client_response_hook=config.client_response_hook_handler,
            default_span_details=config.scope_span_details_extractor,
            excluded_urls=get_excluded_urls(config.exclude_urls_env_key),
            meter=config.meter,
            meter_provider=config.meter_provider,
            server_request_hook=config.server_request_hook_handler,
            tracer_provider=config.tracer_provider,
        )

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """ASGI callable.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function.

        Returns:
            None
        """
        await self.open_telemetry_middleware(scope, receive, send)



from .config import OpenAPIConfig
from .controller import OpenAPIController, OpenAPISchemaResponse
from .datastructures import ResponseSpec

__all__ = ("OpenAPIController", "OpenAPISchemaResponse", "OpenAPIConfig", "ResponseSpec")



from __future__ import annotations

from collections import defaultdict
from functools import lru_cache
from typing import TYPE_CHECKING, Any, Callable, Coroutine, DefaultDict, cast

from litestar._multipart import parse_multipart_form
from litestar._parsers import (
    parse_headers,
    parse_query_string,
    parse_url_encoded_form_data,
)
from litestar.datastructures.upload_file import UploadFile
from litestar.dto.interface import ConnectionContext
from litestar.enums import ParamType, RequestEncodingType
from litestar.exceptions import ValidationException
from litestar.params import BodyKwarg
from litestar.types import Empty

if TYPE_CHECKING:
    from litestar._kwargs import KwargsModel
    from litestar._kwargs.parameter_definition import ParameterDefinition
    from litestar._signature.field import SignatureField
    from litestar.connection import ASGIConnection, Request
    from litestar.dto.interface import DTOInterface

__all__ = (
    "body_extractor",
    "cookies_extractor",
    "create_connection_value_extractor",
    "create_data_extractor",
    "create_multipart_extractor",
    "create_query_default_dict",
    "create_url_encoded_data_extractor",
    "headers_extractor",
    "json_extractor",
    "msgpack_extractor",
    "parse_connection_headers",
    "parse_connection_query_params",
    "query_extractor",
    "request_extractor",
    "scope_extractor",
    "socket_extractor",
    "state_extractor",
)


def create_connection_value_extractor(
    kwargs_model: KwargsModel,
    connection_key: str,
    expected_params: set[ParameterDefinition],
    parser: Callable[[ASGIConnection, KwargsModel], dict[str, Any]] | None = None,
) -> Callable[[dict[str, Any], ASGIConnection], None]:
    """Create a kwargs extractor function.

    Args:
        kwargs_model: The KwargsModel instance.
        connection_key: The attribute key to use.
        expected_params: The set of expected params.
        parser: An optional parser function.

    Returns:
        An extractor function.
    """

    alias_and_key_tuple = tuple(
        (p.field_alias.lower() if p.param_type == ParamType.HEADER else p.field_alias, p.field_name)
        for p in expected_params
    )
    alias_defaults = {
        p.field_alias.lower() if p.param_type == ParamType.HEADER else p.field_alias: p.default_value
        for p in expected_params
        if not (p.is_required or p.default_value is Ellipsis)
    }

    def extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
        data = parser(connection, kwargs_model) if parser else getattr(connection, connection_key, {})

        try:
            connection_mapping: dict[str, Any] = {
                key: data[alias] if alias in data else alias_defaults[alias] for alias, key in alias_and_key_tuple
            }
            values.update(connection_mapping)
        except KeyError as e:
            raise ValidationException(f"Missing required parameter {e.args[0]} for url {connection.url}") from e

    return extractor


@lru_cache(1024)
def create_query_default_dict(
    parsed_query: tuple[tuple[str, str], ...], sequence_query_parameter_names: tuple[str, ...]
) -> DefaultDict[str, list[str] | str]:
    """Transform a list of tuples into a default dict. Ensures non-list values are not wrapped in a list.

    Args:
        parsed_query: The parsed query list of tuples.
        sequence_query_parameter_names: A set of query parameters that should be wrapped in list.

    Returns:
        A default dict
    """
    output: DefaultDict[str, list[str] | str] = defaultdict(list)

    for k, v in parsed_query:
        if k in sequence_query_parameter_names:
            output[k].append(v)  # type: ignore
        else:
            output[k] = v

    return output


def parse_connection_query_params(connection: ASGIConnection, kwargs_model: KwargsModel) -> dict[str, Any]:
    """Parse query params and cache the result in scope.

    Args:
        connection: The ASGI connection instance.
        kwargs_model: The KwargsModel instance.

    Returns:
        A dictionary of parsed values.
    """
    parsed_query = connection.scope["_parsed_query"] = (  # type: ignore
        connection._parsed_query
        if connection._parsed_query is not Empty
        else parse_query_string(connection.scope.get("query_string", b""))
    )
    return create_query_default_dict(
        parsed_query=parsed_query, sequence_query_parameter_names=kwargs_model.sequence_query_parameter_names
    )


def parse_connection_headers(connection: ASGIConnection, _: KwargsModel) -> dict[str, Any]:
    """Parse header parameters and cache the result in scope.

    Args:
        connection: The ASGI connection instance.
        _: The KwargsModel instance.

    Returns:
        A dictionary of parsed values
    """
    parsed_headers = connection.scope["_headers"] = (  # type: ignore
        connection._headers if connection._headers is not Empty else parse_headers(tuple(connection.scope["headers"]))
    )
    return cast("dict[str, Any]", parsed_headers)


def state_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Extract the app state from the connection and insert it to the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["state"] = connection.app.state._state


def headers_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Extract the headers from the connection and insert them to the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["headers"] = connection.headers


def cookies_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Extract the cookies from the connection and insert them to the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["cookies"] = connection.cookies


def query_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Extract the query params from the connection and insert them to the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["query"] = connection.query_params


def scope_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Extract the scope from the connection and insert it into the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["scope"] = connection.scope


def request_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Set the connection instance as the 'request' value in the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["request"] = connection


def socket_extractor(values: dict[str, Any], connection: ASGIConnection) -> None:
    """Set the connection instance as the 'socket' value in the kwargs injected to the handler.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        None
    """
    values["socket"] = connection


def body_extractor(
    values: dict[str, Any],
    connection: Request[Any, Any, Any],
) -> None:
    """Extract the body from the request instance.

    Notes:
        - this extractor sets a Coroutine as the value in the kwargs. These are resolved at a later stage.

    Args:
        connection: The ASGI connection instance.
        values: The kwargs that are extracted from the connection and will be injected into the handler.

    Returns:
        The Body value.
    """
    values["body"] = connection.body()


async def json_extractor(connection: Request[Any, Any, Any]) -> Any:
    """Extract the data from request and insert it into the kwargs injected to the handler.

    Notes:
        - this extractor sets a Coroutine as the value in the kwargs. These are resolved at a later stage.

    Args:
        connection: The ASGI connection instance.

    Returns:
        The JSON value.
    """
    return await connection.json()


async def msgpack_extractor(connection: Request[Any, Any, Any]) -> Any:
    """Extract the data from request and insert it into the kwargs injected to the handler.

    Notes:
        - this extractor sets a Coroutine as the value in the kwargs. These are resolved at a later stage.

    Args:
        connection: The ASGI connection instance.

    Returns:
        The MessagePack value.
    """
    return await connection.msgpack()


def create_multipart_extractor(
    signature_field: SignatureField, is_data_optional: bool, dto_type: type[DTOInterface] | None
) -> Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]:
    """Create a multipart form-data extractor.

    Args:
        signature_field: A SignatureField instance.
        is_data_optional: Boolean dictating whether the field is optional.
        dto_type: The DTO type, if configured for handler.

    Returns:
        An extractor function.
    """
    body_kwarg_multipart_form_part_limit: int | None = None
    if signature_field.kwarg_model and isinstance(signature_field.kwarg_model, BodyKwarg):
        body_kwarg_multipart_form_part_limit = signature_field.kwarg_model.multipart_form_part_limit

    async def extract_multipart(
        connection: Request[Any, Any, Any],
    ) -> Any:
        multipart_form_part_limit = (
            body_kwarg_multipart_form_part_limit
            if body_kwarg_multipart_form_part_limit is not None
            else connection.app.multipart_form_part_limit
        )
        connection.scope["_form"] = form_values = (  # type: ignore[typeddict-unknown-key]
            connection.scope["_form"]  # type: ignore[typeddict-item]
            if "_form" in connection.scope
            else parse_multipart_form(
                body=await connection.body(),
                boundary=connection.content_type[-1].get("boundary", "").encode(),
                multipart_form_part_limit=multipart_form_part_limit,
            )
        )

        if signature_field.is_non_string_sequence:
            return list(form_values.values())
        if signature_field.is_simple_type and signature_field.field_type is UploadFile and form_values:
            return [v for v in form_values.values() if isinstance(v, UploadFile)][0]

        if not form_values and is_data_optional:
            return None

        if dto_type:
            ctx = ConnectionContext.from_connection(connection)
            return dto_type(ctx).builtins_to_data_type(form_values)

        return form_values

    return cast("Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]", extract_multipart)


def create_url_encoded_data_extractor(
    is_data_optional: bool, dto_type: type[DTOInterface] | None
) -> Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]:
    """Create extractor for url encoded form-data.

    Args:
        is_data_optional: Boolean dictating whether the field is optional.
        dto_type: The DTO type, if configured for handler.

    Returns:
        An extractor function.
    """

    async def extract_url_encoded_extractor(
        connection: Request[Any, Any, Any],
    ) -> Any:
        connection.scope["_form"] = form_values = (  # type: ignore[typeddict-unknown-key]
            connection.scope["_form"]  # type: ignore[typeddict-item]
            if "_form" in connection.scope
            else parse_url_encoded_form_data(await connection.body())
        )
        if not form_values and is_data_optional:
            return None

        if dto_type:
            ctx = ConnectionContext.from_connection(connection)
            return dto_type(ctx).builtins_to_data_type(form_values)

        return form_values

    return cast(
        "Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]", extract_url_encoded_extractor
    )


def create_data_extractor(kwargs_model: KwargsModel) -> Callable[[dict[str, Any], ASGIConnection], None]:
    """Create an extractor for a request's body.

    Args:
        kwargs_model: The KwargsModel instance.

    Returns:
        An extractor for the request's body.
    """

    if kwargs_model.expected_form_data:
        media_type, signature_field, dto_type = kwargs_model.expected_form_data

        if media_type == RequestEncodingType.MULTI_PART:
            data_extractor = create_multipart_extractor(
                signature_field=signature_field,
                is_data_optional=kwargs_model.is_data_optional,
                dto_type=dto_type,
            )
        else:
            data_extractor = create_url_encoded_data_extractor(
                is_data_optional=kwargs_model.is_data_optional, dto_type=dto_type
            )
    elif kwargs_model.expected_msgpack_data:
        data_extractor = cast(
            "Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]", msgpack_extractor
        )
    elif kwargs_model.expected_dto_data:
        data_extractor = create_dto_extractor(kwargs_model.expected_dto_data)
    else:
        data_extractor = cast(
            "Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]", json_extractor
        )

    def extractor(
        values: dict[str, Any],
        connection: ASGIConnection[Any, Any, Any, Any],
    ) -> None:
        values["data"] = data_extractor(connection)

    return extractor


def create_dto_extractor(
    dto_type: type[DTOInterface],
) -> Callable[[ASGIConnection[Any, Any, Any, Any]], Coroutine[Any, Any, Any]]:
    """Create a DTO data extractor.

    Args:
        dto_type: The :class:`DTOInterface` subclass.

    Returns:
        An extractor function.
    """

    async def dto_extractor(connection: Request[Any, Any, Any]) -> Any:
        ctx = ConnectionContext.from_connection(connection)
        return dto_type(ctx).bytes_to_data_type(await connection.body())

    return dto_extractor  # type:ignore[return-value]



"""DTO domain types."""
from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import AbstractSet


__all__ = ("DTOConfig",)


@dataclass(frozen=True)
class DTOConfig:
    """Control the generated DTO."""

    exclude: AbstractSet[str] = field(default_factory=set)
    """Explicitly exclude fields from the generated DTO, incompatible with ``include``."""
    rename_fields: dict[str, str] = field(default_factory=dict)
    """Mapping of field names, to new name."""
    max_nested_depth: int = 1
    """The maximum depth of nested items allowed for data transfer."""



from __future__ import annotations

import re
import sys
from typing import Literal, NamedTuple

__all__ = ("Version", "get_version", "parse_version")


if sys.version_info >= (3, 10):
    import importlib.metadata as importlib_metadata
else:
    import importlib_metadata  # pragma: no cover


_ReleaseLevel = Literal["alpha", "beta", "rc", "final"]
_PRE_RELEASE_TAGS = {"alpha", "a", "beta", "b", "rc"}
_PRE_RELEASE_TAGS_CONVERSIONS: dict[str, _ReleaseLevel] = {"a": "alpha", "b": "beta"}

_VERSION_PARTS_RE = re.compile(r"(\d+|[a-z]+|\.)")


class Version(NamedTuple):
    """Litestar version information"""

    major: int
    minor: int
    patch: int
    release_level: _ReleaseLevel
    serial: int

    def formatted(self, short: bool = False) -> str:
        version = f"{self.major}.{self.minor}.{self.patch}"
        if not short:
            version += f"{self.release_level}{self.serial}"
        return version


def parse_version(raw_version: str) -> Version:
    """Parse a version string into a :class:`Version`"""
    parts = [p for p in _VERSION_PARTS_RE.split(raw_version) if p and p != "."]
    release_level: _ReleaseLevel = "final"
    serial = "0"

    if len(parts) == 3:
        major, minor, patch = parts
    elif len(parts) == 5:
        major, minor, patch, release_level, serial = parts  # type: ignore[assignment]
        if release_level not in _PRE_RELEASE_TAGS:
            raise ValueError(f"Invalid release level: {release_level}")
        release_level = _PRE_RELEASE_TAGS_CONVERSIONS.get(release_level, release_level)
    else:
        raise ValueError(f"Invalid version: {raw_version}")

    return Version(
        major=int(major), minor=int(minor), patch=int(patch), release_level=release_level, serial=int(serial)
    )


def get_version() -> Version:
    """Get the version of the installed litestar package"""
    return parse_version(importlib_metadata.version("litestar"))



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any, Generic, TypeVar
from urllib.parse import quote

from litestar import Litestar, MediaType, Request, Response
from litestar.contrib.htmx._utils import HTMX_STOP_POLLING, get_headers
from litestar.contrib.htmx.types import (
    EventAfterType,
    HtmxHeaderType,
    LocationType,
    PushUrlType,
    ReSwapMethod,
    TriggerEventType,
)
from litestar.response import TemplateResponse
from litestar.response_containers import ResponseContainer, Template
from litestar.status_codes import HTTP_200_OK

__all__ = (
    "ClientRedirect",
    "ClientRefresh",
    "HTMXTemplate",
    "HXLocation",
    "HXStopPolling",
    "PushUrl",
    "ReplaceUrl",
    "Reswap",
    "Retarget",
    "TriggerEvent",
)

if TYPE_CHECKING:
    from litestar.background_tasks import BackgroundTask, BackgroundTasks
    from litestar.datastructures import Cookie


# HTMX defined HTTP status code.
# Response carrying this status code will ask client to stop Polling.
T = TypeVar("T")


class HXStopPolling(Response):
    """Stop HTMX client from Polling."""

    def __init__(self) -> None:
        """Initialize"""
        super().__init__(content=None)
        self.status_code = HTMX_STOP_POLLING


class ClientRedirect(Response):
    """HTMX Response class to support client side redirect."""

    def __init__(self, redirect_to: str) -> None:
        """Set status code to 200 (required by HTMX), and pass redirect url."""
        super().__init__(content=None, headers=get_headers(hx_headers=HtmxHeaderType(redirect=redirect_to)))
        del self.headers["Location"]


class ClientRefresh(Response):
    """Response to support HTMX client page refresh"""

    def __init__(self) -> None:
        """Set Status code to 200 and set headers."""
        super().__init__(content=None, headers=get_headers(hx_headers=HtmxHeaderType(refresh=True)))


class PushUrl(Generic[T], Response[T]):
    """Response to push new url into the history stack."""

    def __init__(self, content: T, push_url: PushUrlType, **kwargs: Any) -> None:
        """Initialize PushUrl."""
        super().__init__(
            content=content,
            status_code=HTTP_200_OK,
            headers=get_headers(hx_headers=HtmxHeaderType(push_url=push_url)),
            **kwargs,
        )


class ReplaceUrl(Generic[T], Response[T]):
    """Response to replace url in the Browser Location bar."""

    def __init__(self, content: T, replace_url: PushUrlType, **kwargs: Any) -> None:
        """Initialize ReplaceUrl."""
        super().__init__(
            content=content,
            status_code=HTTP_200_OK,
            headers=get_headers(hx_headers=HtmxHeaderType(replace_url=replace_url)),
            **kwargs,
        )


class Reswap(Generic[T], Response[T]):
    """Response to specify how the response will be swapped."""

    def __init__(
        self,
        content: T,
        method: ReSwapMethod,
        **kwargs: Any,
    ) -> None:
        """Initialize Reswap."""
        super().__init__(content=content, headers=get_headers(hx_headers=HtmxHeaderType(re_swap=method)), **kwargs)


class Retarget(Generic[T], Response[T]):
    """Response to target different element on the page."""

    def __init__(self, content: T, target: str, **kwargs: Any) -> None:
        """Initialize Retarget."""
        super().__init__(content=content, headers=get_headers(hx_headers=HtmxHeaderType(re_target=target)), **kwargs)


class TriggerEvent(Generic[T], Response[T]):
    """Trigger Client side event."""

    def __init__(
        self,
        content: T,
        name: str,
        after: EventAfterType,
        params: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize TriggerEvent."""
        event = TriggerEventType(name=name, params=params, after=after)
        headers = get_headers(hx_headers=HtmxHeaderType(trigger_event=event))
        super().__init__(content=content, headers=headers, **kwargs)


class HXLocation(Response):
    """Client side redirect without full page reload."""

    def __init__(
        self,
        redirect_to: str,
        source: str | None = None,
        event: str | None = None,
        target: str | None = None,
        swap: ReSwapMethod = None,
        hx_headers: dict[str, Any] | None = None,
        values: dict[str, str] | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize HXLocation, Set status code to 200 (required by HTMX),
        and pass redirect url.
        """
        super().__init__(
            content=None,
            headers={"Location": quote(redirect_to, safe="/#%[]=:;$&()+,!?*@'~")},
            **kwargs,
        )
        spec: dict[str, Any] = get_headers(
            hx_headers=HtmxHeaderType(
                location=LocationType(
                    path=str(self.headers.get("Location")),
                    source=source,
                    event=event,
                    target=target,
                    swap=swap,
                    values=values,
                    hx_headers=hx_headers,
                )
            )
        )
        del self.headers["Location"]
        self.headers.update(spec)


@dataclass
class HTMXTemplate(ResponseContainer[TemplateResponse]):
    """HTMX template wrapper"""

    name: str
    """Path-like name for the template to be rendered, e.g. "index.html"."""
    context: dict[str, Any] = field(default_factory=dict)
    """A dictionary of key/value pairs to be passed to the temple engine's render method.

    Defaults to None.
    """
    background: BackgroundTask | BackgroundTasks | None = field(default=None)
    """A :class:`BackgroundTask <.background_tasks.BackgroundTask>` instance or
    :class:`BackgroundTasks <.background_tasks.BackgroundTasks>` to execute after the response is finished. Defaults to
    ``None``.
    """
    headers: dict[str, Any] = field(default_factory=dict)
    """A string/string dictionary of response headers.Header keys are insensitive. Defaults to ``None``."""
    cookies: list[Cookie] = field(default_factory=list)
    """A list of :class:`Cookies <.datastructures.Cookie>` to be set under the response ``Set-Cookie`` header. Defaults
    to ``None``.
    """
    media_type: MediaType | str | None = field(default=None)
    """If defined, overrides the media type configured in the route decorator."""
    encoding: str = field(default="utf-8")
    """The encoding to be used for the response headers."""
    push_url: PushUrlType | None = field(default=None)
    """Either a string value specifying a URL to push to browser history or ``False`` to prevent HTMX client from
    pushing a url to browser history."""
    re_swap: ReSwapMethod | None = field(default=None)
    """Method value to instruct HTMX which swapping method to use."""
    re_target: str | None = field(default=None)
    """Value for 'id of target element' to apply changes to."""
    trigger_event: str | None = field(default=None)
    """Event name to trigger."""
    params: dict[str, Any] | None = field(default=None)
    """Dictionary of parameters if any required with trigger event parameter."""
    after: EventAfterType | None = field(default=None)
    """Changes to apply after ``receive``, ``settle`` or ``swap`` event."""

    def to_response(
        self,
        headers: dict[str, Any],
        media_type: MediaType | str,
        status_code: int,
        app: Litestar,
        request: Request,
    ) -> TemplateResponse:
        """Add HTMX headers and return a :class:`TemplateResponse <.response.TemplateResponse>`."""

        event: TriggerEventType | None = None
        if self.trigger_event:
            event = TriggerEventType(name=str(self.trigger_event), params=self.params, after=self.after)

        hx_headers: dict[str, Any] = get_headers(
            hx_headers=HtmxHeaderType(
                push_url=self.push_url, re_swap=self.re_swap, re_target=self.re_target, trigger_event=event
            )
        )

        template = Template(name=self.name, background=self.background, context=self.context, encoding=self.encoding)

        return template.to_response(
            headers=hx_headers, media_type=media_type, app=app, status_code=status_code, request=request
        )



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Literal

    from typing_extensions import TypeAlias

__all__ = ("ForType",)

ForType: TypeAlias = "Literal['data', 'return']"
"""Type for parameters that express the purpose of DTO application."""



from __future__ import annotations

import inspect
import multiprocessing
import subprocess
from typing import TYPE_CHECKING, Any

import click
from click import command, option
from rich.tree import Tree

from litestar.cli._utils import LitestarEnv, console, show_app_info
from litestar.routes import HTTPRoute, WebSocketRoute
from litestar.utils.helpers import unwrap_partial

__all__ = ("info_command", "routes_command", "run_command")

if TYPE_CHECKING:
    from litestar import Litestar


def _convert_uvicorn_args(args: dict[str, Any]) -> list[str]:
    process_args = []
    for arg, value in args.items():
        if isinstance(value, bool):
            if value:
                process_args.append(f"--{arg}")
        else:
            process_args.append(f"--{arg}={value}")

    return process_args


@command(name="version")
@option("-s", "--short", help="Exclude release level and serial information", is_flag=True, default=False)
def version_command(short: bool) -> None:
    from litestar import __version__

    click.echo(__version__.formatted(short=short))


@command(name="info")
def info_command(app: Litestar) -> None:
    """Show information about the detected Litestar app."""

    show_app_info(app)


@command(name="run")
@option("-r", "--reload", help="Reload server on changes", default=False, is_flag=True)
@option("-p", "--port", help="Serve under this port", type=int, default=8000, show_default=True)
@option(
    "-wc",
    "--web-concurrency",
    help="The number of HTTP workers to launch",
    type=click.IntRange(min=1, max=multiprocessing.cpu_count() + 1),
    show_default=True,
    default=1,
)
@option("--host", help="Server under this host", default="127.0.0.1", show_default=True)
@option("--debug", help="Run app in debug mode", is_flag=True)
def run_command(
    reload: bool,
    port: int,
    web_concurrency: int,
    host: str,
    debug: bool,
    env: LitestarEnv,
    app: Litestar,
) -> None:
    """Run a Litestar app.

    The app can be either passed as a module path in the form of <module name>.<submodule>:<app instance or factory>,
    set as an environment variable LITESTAR_APP with the same format or automatically discovered from one of these
    canonical paths: app.py, asgi.py, application.py or app/__init__.py. When auto-discovering application factories,
    functions with the name ``create_app`` are considered, or functions that are annotated as returning a ``Litestar``
    instance.
    """

    if debug or env.debug:
        app.debug = True

    show_app_info(app)

    console.rule("[yellow]Starting server process", align="left")

    # invoke uvicorn in a subprocess to be able to use the --reload flag. see
    # https://github.com/litestar-org/litestar/issues/1191 and https://github.com/encode/uvicorn/issues/1045

    process_args = {
        "reload": env.reload or reload,
        "host": env.host or host,
        "port": env.port or port,
        "workers": env.web_concurrency or web_concurrency,
        "factory": env.is_app_factory,
    }

    subprocess.run(["uvicorn", env.app_path, *_convert_uvicorn_args(process_args)], check=True)


@command(name="routes")
def routes_command(app: Litestar) -> None:  # pragma: no cover
    """Display information about the application's routes."""

    tree = Tree("", hide_root=True)

    for route in sorted(app.routes, key=lambda r: r.path):
        if isinstance(route, HTTPRoute):
            branch = tree.add(f"[green]{route.path}[/green] (HTTP)")
            for handler in route.route_handlers:
                handler_info = [
                    f"[blue]{handler.name or handler.handler_name}[/blue]",
                ]

                if inspect.iscoroutinefunction(unwrap_partial(handler.fn.value)):
                    handler_info.append("[magenta]async[/magenta]")
                else:
                    handler_info.append("[yellow]sync[/yellow]")

                handler_info.append(f'[cyan]{", ".join(sorted(handler.http_methods))}[/cyan]')

                if len(handler.paths) > 1:
                    for path in handler.paths:
                        branch.add(" ".join([f"[green]{path}[green]", *handler_info]))
                else:
                    branch.add(" ".join(handler_info))

        else:
            route_type = "WS" if isinstance(route, WebSocketRoute) else "ASGI"
            branch = tree.add(f"[green]{route.path}[/green] ({route_type})")
            branch.add(f"[blue]{route.route_handler.name or route.route_handler.handler_name}[/blue]")

    console.print(tree)



from __future__ import annotations

import typing
from inspect import getmodule
from typing import TYPE_CHECKING, TypeVar

from msgspec import Struct
from typing_extensions import get_type_hints

from litestar.utils.signature import ParsedType

from .config import DTOConfig

if TYPE_CHECKING:
    from typing import Any

__all__ = (
    "get_model_type_hints",
    "parse_configs_from_annotation",
)

T = TypeVar("T")
StructT = TypeVar("StructT", bound=Struct)


def get_model_type_hints(model_type: type[Any], namespace: dict[str, Any] | None = None) -> dict[str, ParsedType]:
    """Retrieve type annotations for ``model_type``.

    Args:
        model_type: Any type-annotated class.
        namespace: Optional namespace to use for resolving type hints.

    Returns:
        Parsed type hints for ``model_type`` resolved within the scope of its module.
    """
    namespace = namespace or {}
    namespace.update(vars(typing))
    model_module = getmodule(model_type)
    if model_module:
        namespace.update(vars(model_module))
    return {k: ParsedType(v) for k, v in get_type_hints(model_type, localns=namespace).items()}


def parse_configs_from_annotation(parsed_type: ParsedType) -> tuple[DTOConfig, ...]:
    """Extract data type and config instances from ``Annotated`` annotation.

    Args:
        parsed_type: A parsed type annotation that represents the annotation used to narrow the DTO type.

    Returns:
        The type and config object extracted from the annotation.
    """
    return tuple(item for item in parsed_type.metadata if isinstance(item, DTOConfig))



from __future__ import annotations

from gzip import GzipFile
from io import BytesIO
from typing import TYPE_CHECKING, Any, Literal, Optional

from litestar.constants import SCOPE_STATE_RESPONSE_COMPRESSED
from litestar.datastructures import Headers, MutableScopeHeaders
from litestar.enums import CompressionEncoding, ScopeType
from litestar.exceptions import MissingDependencyException
from litestar.middleware.base import AbstractMiddleware
from litestar.utils import Ref, set_litestar_scope_state

__all__ = ("CompressionFacade", "CompressionMiddleware")


if TYPE_CHECKING:
    from litestar.config.compression import CompressionConfig
    from litestar.types import (
        ASGIApp,
        HTTPResponseStartEvent,
        Message,
        Receive,
        Scope,
        Send,
    )

    try:
        from brotli import Compressor
    except ImportError:
        Compressor = Any


class CompressionFacade:
    """A unified facade offering a uniform interface for different compression libraries."""

    __slots__ = ("compressor", "buffer", "compression_encoding")

    compressor: GzipFile | Compressor  # pyright: ignore

    def __init__(self, buffer: BytesIO, compression_encoding: CompressionEncoding, config: CompressionConfig) -> None:
        """Initialize ``CompressionFacade``.

        Args:
            buffer: A bytes IO buffer to write the compressed data into.
            compression_encoding: The compression encoding used.
            config: The app compression config.
        """
        self.buffer = buffer
        self.compression_encoding = compression_encoding

        if compression_encoding == CompressionEncoding.BROTLI:
            try:
                import brotli  # noqa: F401
            except ImportError as e:
                raise MissingDependencyException("brotli") from e

            from brotli import MODE_FONT, MODE_GENERIC, MODE_TEXT, Compressor

            modes: dict[Literal["generic", "text", "font"], int] = {
                "text": int(MODE_TEXT),
                "font": int(MODE_FONT),
                "generic": int(MODE_GENERIC),
            }
            self.compressor = Compressor(
                quality=config.brotli_quality,
                mode=modes[config.brotli_mode],
                lgwin=config.brotli_lgwin,
                lgblock=config.brotli_lgblock,
            )
        else:
            self.compressor = GzipFile(mode="wb", fileobj=buffer, compresslevel=config.gzip_compress_level)

    def write(self, body: bytes) -> None:
        """Write compressed bytes.

        Args:
            body: Message body to process

        Returns:
            None
        """

        if self.compression_encoding == CompressionEncoding.BROTLI:
            self.buffer.write(self.compressor.process(body) + self.compressor.flush())  # type: ignore
        else:
            self.compressor.write(body)

    def close(self) -> None:
        """Close the compression stream.

        Returns:
            None
        """
        if self.compression_encoding == CompressionEncoding.BROTLI:
            self.buffer.write(self.compressor.finish())  # type: ignore
        else:
            self.compressor.close()


class CompressionMiddleware(AbstractMiddleware):
    """Compression Middleware Wrapper.

    This is a wrapper allowing for generic compression configuration / handler middleware
    """

    def __init__(self, app: ASGIApp, config: CompressionConfig) -> None:
        """Initialize ``CompressionMiddleware``

        Args:
            app: The ``next`` ASGI app to call.
            config: An instance of CompressionConfig.
        """
        super().__init__(
            app=app, exclude=config.exclude, exclude_opt_key=config.exclude_opt_key, scopes={ScopeType.HTTP}
        )
        self.config = config

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """ASGI callable.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function.

        Returns:
            None
        """
        accept_encoding = Headers.from_scope(scope).get("accept-encoding", "")

        if CompressionEncoding.BROTLI in accept_encoding and self.config.backend == "brotli":
            await self.app(
                scope,
                receive,
                self.create_compression_send_wrapper(
                    send=send, compression_encoding=CompressionEncoding.BROTLI, scope=scope
                ),
            )
            return

        if CompressionEncoding.GZIP in accept_encoding and (
            self.config.backend == "gzip" or self.config.brotli_gzip_fallback
        ):
            await self.app(
                scope,
                receive,
                self.create_compression_send_wrapper(
                    send=send, compression_encoding=CompressionEncoding.GZIP, scope=scope
                ),
            )
            return

        await self.app(scope, receive, send)

    def create_compression_send_wrapper(
        self,
        send: Send,
        compression_encoding: Literal[CompressionEncoding.BROTLI, CompressionEncoding.GZIP],
        scope: Scope,
    ) -> Send:
        """Wrap ``send`` to handle brotli compression.

        Args:
            send: The ASGI send function.
            compression_encoding: The compression encoding used.
            scope: The ASGI connection scope

        Returns:
            An ASGI send function.
        """

        buffer = BytesIO()
        facade = CompressionFacade(buffer=buffer, compression_encoding=compression_encoding, config=self.config)

        initial_message = Ref[Optional["HTTPResponseStartEvent"]](None)
        started = Ref[bool](False)

        async def send_wrapper(message: Message) -> None:
            """Handle and compresses the HTTP Message with brotli.

            Args:
                message (Message): An ASGI Message.
            """

            if message["type"] == "http.response.start":
                initial_message.value = message
                return

            if initial_message.value and message["type"] == "http.response.body":
                body = message["body"]
                more_body = message.get("more_body")

                if not started.value:
                    started.value = True
                    if more_body:
                        headers = MutableScopeHeaders(initial_message.value)
                        headers["Content-Encoding"] = compression_encoding
                        headers.extend_header_value("vary", "Accept-Encoding")
                        del headers["Content-Length"]
                        set_litestar_scope_state(scope, SCOPE_STATE_RESPONSE_COMPRESSED, True)

                        facade.write(body)

                        message["body"] = buffer.getvalue()
                        buffer.seek(0)
                        buffer.truncate()
                        await send(initial_message.value)
                        await send(message)

                    elif len(body) >= self.config.minimum_size:
                        facade.write(body)
                        facade.close()
                        body = buffer.getvalue()

                        headers = MutableScopeHeaders(initial_message.value)
                        headers["Content-Encoding"] = compression_encoding
                        headers["Content-Length"] = str(len(body))
                        headers.extend_header_value("vary", "Accept-Encoding")
                        message["body"] = body
                        set_litestar_scope_state(scope, SCOPE_STATE_RESPONSE_COMPRESSED, True)

                        await send(initial_message.value)
                        await send(message)

                    else:
                        await send(initial_message.value)
                        await send(message)

                else:
                    facade.write(body)
                    if not more_body:
                        facade.close()

                    message["body"] = buffer.getvalue()

                    buffer.seek(0)
                    buffer.truncate()

                    if not more_body:
                        buffer.close()

                    await send(message)

        return send_wrapper



from __future__ import annotations

from datetime import timedelta
from typing import cast

from redis.asyncio import Redis
from redis.asyncio.connection import ConnectionPool

from litestar.exceptions import ImproperlyConfiguredException
from litestar.types import Empty, EmptyType

from .base import NamespacedStore

__all__ = ("RedisStore",)


class RedisStore(NamespacedStore):
    """Redis based, thread and process safe asynchronous key/value store."""

    __slots__ = ("_redis",)

    def __init__(self, redis: Redis, namespace: str | None | EmptyType = Empty) -> None:
        """Initialize :class:`RedisStore`

        Args:
            redis: An :class:`redis.asyncio.Redis` instance
            namespace: A key prefix to simulate a namespace in redis. If not given,
                defaults to ``LITESTAR``. Namespacing can be explicitly disabled by passing
                ``None``. This will make :meth:`.delete_all` unavailable.
        """
        self._redis = redis
        self.namespace: str | None = "LITESTAR" if namespace is Empty else namespace  # type: ignore[assignment]

        # script to get and renew a key in one atomic step
        self._get_and_renew_script = self._redis.register_script(
            b"""
        local key = KEYS[1]
        local renew = tonumber(ARGV[1])

        local data = redis.call('GET', key)
        local ttl = redis.call('TTL', key)

        if ttl > 0 then
            redis.call('EXPIRE', key, renew)
        end

        return data
        """
        )

        # script to delete all keys in the namespace
        self._delete_all_script = self._redis.register_script(
            b"""
        local cursor = 0

        repeat
            local result = redis.call('SCAN', cursor, 'MATCH', ARGV[1])
            for _,key in ipairs(result[2]) do
                redis.call('UNLINK', key)
            end
            cursor = tonumber(result[1])
        until cursor == 0
        """
        )

    @classmethod
    def with_client(
        cls,
        url: str = "redis://localhost:6379",
        *,
        db: int | None = None,
        port: int | None = None,
        username: str | None = None,
        password: str | None = None,
        namespace: str | None | EmptyType = Empty,
    ) -> RedisStore:
        """Initialize a :class:`RedisStore` instance with a new class:`redis.asyncio.Redis` instance.

        Args:
            url: Redis URL to connect to
            db: Redis database to use
            port: Redis port to use
            username: Redis username to use
            password: Redis password to use
            namespace: Virtual key namespace to use
        """
        pool = ConnectionPool.from_url(
            url=url,
            db=db,
            decode_responses=False,
            port=port,
            username=username,
            password=password,
        )
        return cls(redis=Redis(connection_pool=pool), namespace=namespace)

    def with_namespace(self, namespace: str) -> RedisStore:
        """Return a new :class:`RedisStore` with a nested virtual key namespace.
        The current instances namespace will serve as a prefix for the namespace, so it
        can be considered the parent namespace.
        """
        return type(self)(redis=self._redis, namespace=f"{self.namespace}_{namespace}" if self.namespace else namespace)

    def _make_key(self, key: str) -> str:
        prefix = f"{self.namespace}:" if self.namespace else ""
        return prefix + key

    async def set(self, key: str, value: str | bytes, expires_in: int | timedelta | None = None) -> None:
        """Set a value.

        Args:
            key: Key to associate the value with
            value: Value to store
            expires_in: Time in seconds before the key is considered expired

        Returns:
            ``None``
        """
        if isinstance(value, str):
            value = value.encode("utf-8")
        await self._redis.set(self._make_key(key), value, ex=expires_in)

    async def get(self, key: str, renew_for: int | timedelta | None = None) -> bytes | None:
        """Get a value.

        Args:
            key: Key associated with the value
            renew_for: If given and the value had an initial expiry time set, renew the
                expiry time for ``renew_for`` seconds. If the value has not been set
                with an expiry time this is a no-op. Atomicity of this step is guaranteed
                by using a lua script to execute fetch and renewal. If ``renew_for`` is
                not given, the script will be bypassed so no overhead will occur

        Returns:
            The value associated with ``key`` if it exists and is not expired, else
            ``None``
        """
        key = self._make_key(key)
        if renew_for:
            if isinstance(renew_for, timedelta):
                renew_for = renew_for.seconds
            data = await self._get_and_renew_script(keys=[key], args=[renew_for])
            return cast("bytes | None", data)
        return await self._redis.get(key)

    async def delete(self, key: str) -> None:
        """Delete a value.

        If no such key exists, this is a no-op.

        Args:
            key: Key of the value to delete
        """
        await self._redis.delete(self._make_key(key))

    async def delete_all(self) -> None:
        """Delete all stored values in the virtual key namespace.

        Raises:
            ImproperlyConfiguredException: If no namespace was configured
        """
        if not self.namespace:
            raise ImproperlyConfiguredException("Cannot perform delete operation: No namespace configured")

        await self._delete_all_script(keys=[], args=[f"{self.namespace}*:*"])

    async def exists(self, key: str) -> bool:
        """Check if a given ``key`` exists."""
        return await self._redis.exists(self._make_key(key)) == 1

    async def expires_in(self, key: str) -> int | None:
        """Get the time in seconds ``key`` expires in. If no such ``key`` exists or no
        expiry time was set, return ``None``.
        """
        ttl = await self._redis.ttl(self._make_key(key))
        if ttl == -2:
            return None
        return ttl



from __future__ import annotations

from inspect import cleandoc
from typing import TYPE_CHECKING

from litestar._openapi.parameters import create_parameter_for_handler
from litestar._openapi.request_body import create_request_body
from litestar._openapi.responses import create_responses
from litestar._openapi.utils import SEPARATORS_CLEANUP_PATTERN
from litestar.openapi.spec.operation import Operation
from litestar.openapi.spec.path_item import PathItem
from litestar.utils.helpers import unwrap_partial

__all__ = ("create_path_item", "extract_layered_values", "get_description_for_handler")


if TYPE_CHECKING:
    from litestar.handlers.http_handlers import HTTPRouteHandler
    from litestar.openapi.spec import Schema, SecurityRequirement
    from litestar.plugins import OpenAPISchemaPluginProtocol
    from litestar.routes import HTTPRoute
    from litestar.types.callable_types import OperationIDCreator


def get_description_for_handler(route_handler: HTTPRouteHandler, use_handler_docstrings: bool) -> str | None:
    """Produce the operation description for a route handler, either by using the description value if provided,

    or the docstring - if config is enabled.

    Args:
        route_handler: A route handler instance.
        use_handler_docstrings: If ``True`` and `route_handler.description`` is ``None` returns docstring of wrapped
            handler function.

    Returns:
        An optional description string
    """
    handler_description = route_handler.description
    if handler_description is None and use_handler_docstrings:
        fn = unwrap_partial(route_handler.fn.value)
        return cleandoc(fn.__doc__) if fn.__doc__ else None
    return handler_description


def extract_layered_values(
    route_handler: HTTPRouteHandler,
) -> tuple[list[str] | None, list[dict[str, list[str]]] | None]:
    """Extract the tags and security values from the route handler layers.

    Args:
        route_handler: A Route Handler instance.

    Returns:
        A tuple of optional lists.
    """
    tags: list[str] = []
    security: list[SecurityRequirement] = []
    for layer in route_handler.ownership_layers:
        if layer.tags:
            tags.extend(layer.tags)
        if layer.security:
            security.extend(layer.security)
    return sorted(set(tags)) if tags else None, security or None


def create_path_item(
    create_examples: bool,
    operation_id_creator: OperationIDCreator,
    plugins: list[OpenAPISchemaPluginProtocol],
    route: HTTPRoute,
    schemas: dict[str, Schema],
    use_handler_docstrings: bool,
) -> tuple[PathItem, list[str]]:
    """Create a PathItem for the given route parsing all http_methods into Operation Models.

    Args:
        create_examples: Whether to auto-generate examples.
        operation_id_creator: A function to generate operation ids.
        plugins: A list of plugins.
        route: An HTTPRoute instance.
        schemas: A mapping of schemas.
        use_handler_docstrings: Whether to use the handler docstring.

    Returns:
        A tuple containing the path item and a list of operation ids.
    """
    path_item = PathItem()
    operation_ids: list[str] = []

    for http_method, handler_tuple in route.route_handler_map.items():
        route_handler, _ = handler_tuple

        if route_handler.include_in_schema:
            handler_fields = route_handler.signature_model.fields if route_handler.signature_model else {}
            parameters = (
                create_parameter_for_handler(
                    route_handler=route_handler,
                    handler_fields=handler_fields,
                    path_parameters=route.path_parameters,
                    generate_examples=create_examples,
                    schemas=schemas,
                )
                or None
            )
            raises_validation_error = bool("data" in handler_fields or path_item.parameters or parameters)

            request_body = None
            if "data" in handler_fields:
                request_body = create_request_body(
                    route_handler=route_handler,
                    field=handler_fields["data"],
                    generate_examples=create_examples,
                    plugins=plugins,
                    schemas=schemas,
                )
            operation_id = route_handler.operation_id or operation_id_creator(
                route_handler, http_method, route.path_components
            )
            tags, security = extract_layered_values(route_handler)
            operation = Operation(
                operation_id=operation_id,
                tags=tags,
                summary=route_handler.summary or SEPARATORS_CLEANUP_PATTERN.sub("", route_handler.handler_name.title()),
                description=get_description_for_handler(route_handler, use_handler_docstrings),
                deprecated=route_handler.deprecated,
                responses=create_responses(
                    route_handler=route_handler,
                    raises_validation_error=raises_validation_error,
                    generate_examples=create_examples,
                    plugins=plugins,
                    schemas=schemas,
                ),
                request_body=request_body,
                parameters=parameters,  # type: ignore[arg-type]
                security=security,
            )
            operation_ids.append(operation_id)
            setattr(path_item, http_method.lower(), operation)

    return path_item, operation_ids



from __future__ import annotations

from dataclasses import asdict
from typing import TYPE_CHECKING, Any

from pydantic import BaseConfig, BaseModel, ValidationError, create_model
from pydantic.fields import FieldInfo, ModelField

from litestar._signature.field import SignatureField
from litestar._signature.models.base import SignatureModel
from litestar.constants import UNDEFINED_SENTINELS
from litestar.params import BodyKwarg, DependencyKwarg, ParameterKwarg
from litestar.types import Empty
from litestar.utils.predicates import is_pydantic_constrained_field

if TYPE_CHECKING:
    from litestar.connection import ASGIConnection
    from litestar.utils.signature import ParsedSignature

__all__ = ("PydanticSignatureModel",)


class PydanticSignatureModel(SignatureModel, BaseModel):
    """Model that represents a function signature that uses a pydantic specific type or types."""

    class Config(BaseConfig):
        copy_on_model_validation = "none"
        arbitrary_types_allowed = True

    @classmethod
    def parse_values_from_connection_kwargs(cls, connection: ASGIConnection, **kwargs: Any) -> dict[str, Any]:
        """Extract values from the connection instance and return a dict of parsed values.

        Args:
            connection: The ASGI connection instance.
            **kwargs: A dictionary of kwargs.

        Raises:
            ValidationException: If validation failed.
            InternalServerException: If another exception has been raised.

        Returns:
            A dictionary of parsed values
        """
        try:
            signature = cls(**kwargs)
        except ValidationError as e:
            raise cls._create_exception(
                messages=[{"key": str(exc["loc"][-1]), "message": exc["msg"]} for exc in e.errors()],
                connection=connection,
            ) from e

        return signature.to_dict()

    def to_dict(self) -> dict[str, Any]:
        """Normalize access to the signature model's dictionary method, because different backends use different methods
        for this.

        Returns: A dictionary of string keyed values.
        """
        return {key: self.__getattribute__(key) for key in self.__fields__}

    @classmethod
    def signature_field_from_model_field(cls, model_field: ModelField) -> SignatureField:
        """Create a SignatureField instance from a pydantic ModelField.

        Args:
            model_field: A pydantic ModelField instance.

        Returns:
            A SignatureField
        """
        children = (
            tuple(cls.signature_field_from_model_field(sub_field) for sub_field in model_field.sub_fields)
            if model_field.sub_fields
            else None
        )
        default_value = (
            model_field.field_info.default if model_field.field_info.default not in UNDEFINED_SENTINELS else Empty
        )

        kwarg_model: ParameterKwarg | DependencyKwarg | BodyKwarg | None = model_field.field_info.extra.pop(
            "kwargs_model", None
        )
        if kwarg_model:
            default_value = kwarg_model.default
        elif isinstance(default_value, (ParameterKwarg, DependencyKwarg, BodyKwarg)):
            kwarg_model = default_value
            default_value = default_value.default

        return SignatureField(
            children=children,
            default_value=default_value,
            extra=model_field.field_info.extra or {},
            field_type=model_field.annotation if model_field.annotation is not Empty else Any,
            kwarg_model=kwarg_model,
            name=model_field.name,
        )

    @classmethod
    def populate_signature_fields(cls) -> None:
        """Populate the class signature fields.

        Returns:
            None.
        """
        cls.fields = {k: cls.signature_field_from_model_field(v) for k, v in cls.__fields__.items()}

    @classmethod
    def create(
        cls,
        fn_name: str,
        fn_module: str | None,
        parsed_signature: ParsedSignature,
        dependency_names: set[str],
        type_overrides: dict[str, Any],
    ) -> type[PydanticSignatureModel]:
        """Create a pydantic based SignatureModel.

        Args:
            fn_name: Name of the callable.
            fn_module: Name of the function's module, if any.
            parsed_signature: A ParsedSignature instance.
            dependency_names: A set of dependency names.
            type_overrides: A dictionary of type overrides, either will override a parameter type with a type derived
                from a plugin, or set the type to ``Any`` if validation should be skipped for the parameter.

        Returns:
            The created PydanticSignatureModel.
        """
        field_definitions: dict[str, tuple[Any, Any]] = {}

        for parameter in parsed_signature.parameters.values():
            field_type = type_overrides.get(parameter.name, parameter.parsed_type.annotation)

            if kwargs_container := parameter.kwarg_container:
                if isinstance(kwargs_container, DependencyKwarg):
                    field_info = FieldInfo(
                        default=kwargs_container.default if kwargs_container.default is not Empty else None,
                        kwargs_model=kwargs_container,
                        parsed_parameter=parameter,
                    )
                    if kwargs_container.skip_validation:
                        field_type = Any
                else:
                    field_info = FieldInfo(
                        **{k: v for k, v in asdict(kwargs_container).items() if v is not Empty},
                        kwargs_model=kwargs_container,
                        parsed_parameter=parameter,
                    )
            else:
                field_info = FieldInfo(default=..., parsed_parameter=parameter)

                if is_pydantic_constrained_field(parameter.default):
                    field_type = parameter.default
                elif parameter.has_default:
                    field_info.default = parameter.default
                elif parameter.parsed_type.is_optional:
                    field_info.default = None

            field_definitions[parameter.name] = (field_type, field_info)

        model: type[PydanticSignatureModel] = create_model(  # type: ignore
            f"{fn_name}_signature_model",
            __base__=PydanticSignatureModel,
            __module__=fn_module or "pydantic.main",
            **field_definitions,
        )
        model.return_annotation = parsed_signature.return_type.annotation
        model.dependency_name_set = dependency_names
        model.populate_signature_fields()
        return model



from __future__ import annotations

from collections.abc import Collection as CollectionsCollection
from typing import TYPE_CHECKING, NewType, TypeVar

from msgspec import Struct, defstruct, field

from litestar.dto.factory.types import NestedFieldDefinition
from litestar.dto.factory.utils import get_model_type_hints
from litestar.types import Empty

if TYPE_CHECKING:
    from typing import Any, Collection

    from litestar.dto.factory.types import FieldDefinition, FieldDefinitionsType
    from litestar.utils.signature import ParsedType


MsgspecField = NewType("MsgspecField", type)
StructT = TypeVar("StructT", bound=Struct)
T = TypeVar("T")


def _create_msgspec_field(field_definition: FieldDefinition) -> MsgspecField | None:
    kws: dict[str, Any] = {}
    if field_definition.default is not Empty:
        kws["default"] = field_definition.default

    if field_definition.default_factory is not Empty:
        kws["default_factory"] = field_definition.default_factory

    if not kws:
        return None

    return field(**kws)  # type:ignore[no-any-return]


def _create_struct_field_def(
    name: str, type_: type[Any], field_: MsgspecField | None
) -> tuple[str, type[Any]] | tuple[str, type[Any], MsgspecField]:
    if field_ is None:
        return name, type_
    return name, type_, field_


def _create_struct_for_field_definitions(model_name: str, field_definitions: FieldDefinitionsType) -> type[Struct]:
    struct_fields: list[tuple[str, type] | tuple[str, type, MsgspecField]] = []
    for _k, v in field_definitions.items():
        field_name = v.serialization_name or v.name
        if isinstance(v, NestedFieldDefinition):
            nested_struct = _create_struct_for_field_definitions(
                f"{model_name}.{field_name}", v.nested_field_definitions
            )
            struct_fields.append(
                _create_struct_field_def(
                    field_name, v.make_field_type(nested_struct), _create_msgspec_field(v.field_definition)
                )
            )
        else:
            struct_fields.append(
                _create_struct_field_def(field_name, v.parsed_type.annotation, _create_msgspec_field(v))
            )
    return defstruct(model_name, struct_fields, frozen=True, kw_only=True)


def _build_model_from_struct(
    model_type: type[T], data: Struct, field_definitions: FieldDefinitionsType, reverse_name_map: dict[str, str]
) -> T:
    """Create instance of ``model_type``.

    Args:
        model_type: the model type received by the DTO on type narrowing.
        data: primitive data that has been parsed and validated via the backend.
        field_definitions: model field definitions.
        reverse_name_map: reverse name map for field definitions.

    Returns:
        Data parsed into ``model_type``.
    """
    unstructured_data = {}
    for k in data.__slots__:  # type:ignore[attr-defined]
        v = getattr(data, k)
        field_name = reverse_name_map.get(k, k)
        field = field_definitions[field_name]

        if isinstance(field, NestedFieldDefinition) and isinstance(v, CollectionsCollection):
            parsed_type = field.field_definition.parsed_type
            if parsed_type.origin is None:  # pragma: no cover
                raise RuntimeError("Unexpected origin value for collection type.")
            unstructured_data[field_name] = parsed_type.origin(
                _build_model_from_struct(field.nested_type, item, field.nested_field_definitions, reverse_name_map)
                for item in v
            )
        elif isinstance(field, NestedFieldDefinition) and isinstance(v, Struct):
            unstructured_data[field_name] = _build_model_from_struct(
                field.nested_type, v, field.nested_field_definitions, reverse_name_map
            )
        else:
            unstructured_data[field_name] = v

    return model_type(**unstructured_data)


def _build_data_from_struct(
    model_type: type[T],
    data: Struct | Collection[Struct],
    field_definitions: FieldDefinitionsType,
    reverse_name_map: dict[str, str],
) -> T | Collection[T]:
    """Create instance or iterable of instances of ``model_type``.

    Args:
        model_type: the model type received by the DTO on type narrowing.
        data: primitive data that has been parsed and validated via the backend.
        field_definitions: model field definitions.
        reverse_name_map: reverse name map for field definitions.

    Returns:
        Data parsed into ``model_type``.
    """
    if isinstance(data, CollectionsCollection):
        return type(data)(  # type:ignore[return-value]
            _build_data_from_struct(model_type, item, field_definitions, reverse_name_map)  # type:ignore[call-arg]
            for item in data
        )
    return _build_model_from_struct(model_type, data, field_definitions, reverse_name_map)


def _build_struct_from_model(model: Any, struct_type: type[StructT], reverse_name_map: dict[str, str]) -> StructT:
    """Convert ``model`` to instance of ``struct_type``

    It is expected that attributes of ``struct_type`` are a subset of the attributes of ``model``.

    Args:
        model: a model instance
        struct_type: a subclass of ``msgspec.Struct``
        reverse_name_map: reverse name map for field definitions.

    Returns:
        Instance of ``struct_type``.
    """
    data = {}
    for key, parsed_type in get_model_type_hints(struct_type).items():
        model_name = reverse_name_map.get(key, key)
        model_val = getattr(model, model_name)
        if parsed_type.is_subclass_of(Struct):
            data[key] = _build_struct_from_model(model_val, parsed_type.annotation, {})
        elif parsed_type.is_union:
            data[key] = _handle_union_type(parsed_type, model_val)
        elif parsed_type.is_collection:
            data[key] = _handle_collection_type(parsed_type, model_val)
        else:
            data[key] = model_val
    return struct_type(**data)


def _handle_union_type(parsed_type: ParsedType, model_val: Any) -> Any:
    """Handle union type.

    Args:
        parsed_type: Parsed type.
        model_val: Model value.

    Returns:
        Model value.
    """
    for inner_type in parsed_type.inner_types:
        if inner_type.is_subclass_of(Struct):
            # If there are multiple struct inner types, we use the first one that creates without exception.
            # This is suboptimal, and perhaps can be improved by assigning the model that the inner struct
            # was derived upon to the struct itself, which would allow us to isolate the correct struct to use
            # for the nested model type instance. For the most likely case of an optional union of a single
            # nested type, this should be sufficient.
            try:
                return _build_struct_from_model(model_val, inner_type.annotation, {})
            except (AttributeError, TypeError):
                continue
    return model_val


def _handle_collection_type(parsed_type: ParsedType, model_val: Any) -> Any:
    """Handle collection type.

    Args:
        parsed_type: Parsed type.
        model_val: Model value.

    Returns:
        Model value.
    """
    if parsed_type.inner_types and (inner_type := parsed_type.inner_types[0]).is_subclass_of(Struct):
        return parsed_type.origin(_build_struct_from_model(m, inner_type.annotation, {}) for m in model_val)
    return model_val



from __future__ import annotations

import re
import traceback
from dataclasses import asdict
from datetime import date, datetime, time, timedelta, timezone
from functools import lru_cache, partial
from pathlib import PurePath
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    cast,
)
from uuid import UUID

from _decimal import Decimal
from typing_extensions import get_args

from litestar._signature.field import SignatureField
from litestar._signature.models.base import ErrorMessage, SignatureModel
from litestar.connection import ASGIConnection, Request, WebSocket
from litestar.datastructures import ImmutableState, MultiDict, State, UploadFile
from litestar.exceptions import MissingDependencyException
from litestar.params import BodyKwarg, DependencyKwarg, ParameterKwarg
from litestar.types import Empty
from litestar.utils import compact
from litestar.utils.predicates import is_optional_union, is_union
from litestar.utils.typing import get_origin_or_inner_type, make_non_optional_union, unwrap_union

try:
    import attr
    import attrs
    import cattrs
except ImportError as e:
    raise MissingDependencyException("attrs") from e

try:
    from dateutil.parser import parse
except ImportError as e:
    raise MissingDependencyException("python-dateutil", "attrs") from e

try:
    from pytimeparse.timeparse import timeparse
except ImportError as e:
    raise MissingDependencyException("pytimeparse", "attrs") from e

if TYPE_CHECKING:
    from litestar.utils.signature import ParsedSignature

__all__ = ("AttrsSignatureModel",)
key_re = re.compile("@ attribute (.*)|'(.*)'")
TRUE_SET = {"1", "true", "on", "t", "y", "yes"}
FALSE_SET = {"0", "false", "off", "f", "n", "no"}

try:
    import pydantic

    def _structure_base_model(value: Any, cls: type[pydantic.BaseModel]) -> pydantic.BaseModel:
        if isinstance(value, pydantic.BaseModel):
            return value
        return cls(**value)

    pydantic_hooks: list[tuple[type[Any], Callable[[Any, type[Any]], Any]]] = [
        (pydantic.BaseModel, _structure_base_model),
    ]
except ImportError:
    pydantic_hooks = []


def _pass_through_structure_hook(value: Any, _: type[Any]) -> Any:
    return value


def _pass_through_unstructure_hook(value: Any) -> Any:
    return value


def _structure_bool(value: Any, _: type[bool]) -> bool:
    if isinstance(value, bytes):
        value = value.decode("utf-8").lower()

    if isinstance(value, str):
        value = value.lower()

    if value == 0 or value in FALSE_SET:
        return False

    if value == 1 or value in TRUE_SET:
        return True

    raise ValueError(f"Cannot convert {value} to bool")


def _structure_datetime(value: Any, cls: type[datetime]) -> datetime:
    if isinstance(value, datetime):
        return value

    try:
        return cls.fromtimestamp(float(value), tz=timezone.utc)
    except (ValueError, TypeError):
        pass

    return parse(value)


def _structure_date(value: Any, cls: type[date]) -> date:
    if isinstance(value, date) and not isinstance(value, datetime):
        return value

    if isinstance(value, (float, int, Decimal)):
        return datetime.fromtimestamp(float(value), tz=timezone.utc).date()

    dt = _structure_datetime(value=value, cls=datetime)
    return cls(year=dt.year, month=dt.month, day=dt.day)


def _structure_time(value: Any, cls: type[time]) -> time:
    if isinstance(value, time):
        return value

    if isinstance(value, str):
        return cls.fromisoformat(value)

    dt = _structure_datetime(value=value, cls=datetime)
    return cls(hour=dt.hour, minute=dt.minute, second=dt.second, microsecond=dt.microsecond, tzinfo=dt.tzinfo)


def _structure_timedelta(value: Any, cls: type[timedelta]) -> timedelta:
    if isinstance(value, timedelta):
        return value
    if isinstance(value, (float, int, Decimal)):
        return cls(seconds=int(value))
    return cls(seconds=timeparse(value))


def _structure_decimal(value: Any, cls: type[Decimal]) -> Decimal:
    return cls(str(value))


def _structure_path(value: Any, cls: type[PurePath]) -> PurePath:
    return cls(str(value))


def _structure_uuid(value: Any, cls: type[UUID]) -> UUID:
    return value if isinstance(value, UUID) else cls(str(value))


def _structure_multidict(value: Any, cls: type[MultiDict]) -> MultiDict:
    return cls(value)


def _structure_str(value: Any, cls: type[str]) -> str:
    # see: https://github.com/python-attrs/cattrs/issues/26#issuecomment-358594015
    if value is None:
        raise ValueError
    return cls(value)


hooks: list[tuple[type[Any], Callable[[Any, type[Any]], Any]]] = [
    (ASGIConnection, _pass_through_structure_hook),
    (Decimal, _structure_decimal),
    (ImmutableState, _pass_through_structure_hook),
    (MultiDict, _structure_multidict),
    (PurePath, _structure_path),
    (Request, _pass_through_structure_hook),
    (State, _pass_through_structure_hook),
    (UUID, _structure_uuid),
    (UploadFile, _pass_through_structure_hook),
    (WebSocket, _pass_through_structure_hook),
    (bool, _structure_bool),
    (date, _structure_date),
    (datetime, _structure_datetime),
    (str, _structure_str),
    (time, _structure_time),
    (timedelta, _structure_timedelta),
    *pydantic_hooks,
]


def _create_default_structuring_hooks(
    converter: cattrs.Converter,
) -> tuple[Callable, Callable]:
    """Create scoped default hooks for a given converter.

    Notes:
        - We are forced to use this pattern because some types cannot be handled by cattrs out of the box. For example,
            union types, optionals, complex union types etc.
        - See: https://github.com/python-attrs/cattrs/issues/311
    Args:
        converter: A converter instance

    Returns:
        A tuple of hook handlers
    """

    @lru_cache(1024)
    def _default_structuring_hook(value: Any, annotation: Any) -> Any:
        for arg in unwrap_union(annotation) or get_args(annotation):
            try:
                return converter.structure(arg, value)
            except ValueError:  # pragma: no cover
                continue
        return value

    return (
        _pass_through_unstructure_hook,
        _default_structuring_hook,
    )


class Converter(cattrs.Converter):
    def __init__(self) -> None:
        super().__init__()

        # this is a hack to create a catch-all hook, see: https://github.com/python-attrs/cattrs/issues/311
        self._structure_func._function_dispatch._handler_pairs[-1] = (
            *_create_default_structuring_hooks(self),
            False,
        )

        for cls, structure_hook in hooks:
            self.register_structure_hook(cls, structure_hook)
            self.register_unstructure_hook(cls, _pass_through_unstructure_hook)


_converter: Converter = Converter()


def _extract_exceptions(e: Any) -> list[ErrorMessage]:
    """Extracts and normalizes cattrs exceptions.

    Args:
        e: An ExceptionGroup - which is a py3.11 feature. We use hasattr instead of instance checks to avoid installing this.

    Returns:
        A list of normalized exception messages.
    """
    messages: list[ErrorMessage] = []
    if hasattr(e, "exceptions"):
        for exc in cast("list[Exception]", e.exceptions):
            if hasattr(exc, "exceptions"):  # pragma: no cover
                # cattrs raises an exception group, where each exception can potentially be an exception group.
                # this case is not reproducible in any of our tests - and frankly I have no idea when it would occur,
                # so this clause is defensive programming only.
                messages.extend(_extract_exceptions(exc))
                continue
            if err_format := [
                line
                for line in traceback.format_exception(type(exc), value=exc, tb=exc.__traceback__)
                if key_re.search(line)
            ]:
                messages.append({"key": key_re.findall(compact(err_format)[-1])[0][0].strip(), "message": str(exc)})
    return messages


def _create_validators(
    annotation: Any, kwargs_model: BodyKwarg | ParameterKwarg
) -> list[Callable[[Any, attrs.Attribute[Any], Any], Any]] | Callable[[Any, attrs.Attribute[Any], Any], Any]:
    validators: list[Callable[[Any, attrs.Attribute[Any], Any], Any]] = []

    for value, validator in [
        (kwargs_model.gt, attrs.validators.gt),
        (kwargs_model.ge, attrs.validators.ge),
        (kwargs_model.lt, attrs.validators.lt),
        (kwargs_model.le, attrs.validators.le),
        (kwargs_model.min_length, attrs.validators.min_len),
        (kwargs_model.max_length, attrs.validators.max_len),
        (kwargs_model.min_items, attrs.validators.min_len),
        (kwargs_model.max_items, attrs.validators.max_len),
        (kwargs_model.regex, partial(attrs.validators.matches_re, flags=0)),
    ]:
        if value is not None:
            validators.append(validator(value))  # type: ignore

    if is_optional_union(annotation):
        annotation = make_non_optional_union(annotation)
        instance_of_validator = attrs.validators.instance_of(
            unwrap_union(annotation) if is_union(annotation) else (get_origin_or_inner_type(annotation) or annotation)
        )
        return attrs.validators.optional([instance_of_validator, *validators])

    instance_of_validator = attrs.validators.instance_of(get_origin_or_inner_type(annotation) or annotation)
    return [instance_of_validator, *validators]


@attr.define
class AttrsSignatureModel(SignatureModel):
    """Model that represents a function signature that uses a pydantic specific type or types."""

    @classmethod
    def parse_values_from_connection_kwargs(cls, connection: ASGIConnection, **kwargs: Any) -> dict[str, Any]:
        try:
            signature = _converter.structure(obj=kwargs, cl=cls)
        except (cattrs.ClassValidationError, ValueError, TypeError, AttributeError) as e:
            raise cls._create_exception(messages=_extract_exceptions(e), connection=connection) from e

        return cast("dict[str, Any]", _converter.unstructure(obj=signature))

    def to_dict(self) -> dict[str, Any]:
        return attrs.asdict(self)

    @classmethod
    def populate_signature_fields(cls) -> None:
        cls.fields = {
            k: SignatureField.create(
                field_type=attribute.type,
                name=k,
                default_value=attribute.default if attribute.default is not attr.NOTHING else Empty,
                kwarg_model=attribute.metadata.get("kwargs_model", None) if attribute.metadata else None,
                extra=attribute.metadata or None,
            )
            for k, attribute in attrs.fields_dict(cls).items()
        }

    @classmethod
    def create(
        cls,
        fn_name: str,
        fn_module: str | None,
        parsed_signature: ParsedSignature,
        dependency_names: set[str],
        type_overrides: dict[str, Any],
    ) -> type[SignatureModel]:
        attributes: dict[str, Any] = {}

        for parameter in parsed_signature.parameters.values():
            annotation = type_overrides.get(parameter.name, parameter.parsed_type.annotation)

            if kwargs_container := parameter.kwarg_container:
                if isinstance(kwargs_container, DependencyKwarg):
                    attribute = attr.attrib(
                        type=annotation if not kwargs_container.skip_validation else Any,
                        default=kwargs_container.default if kwargs_container.default is not Empty else None,
                        metadata={
                            "kwargs_model": kwargs_container,
                            "parsed_parameter": parameter,
                        },
                    )
                else:
                    attribute = attr.attrib(
                        type=annotation,
                        metadata={
                            **asdict(kwargs_container),
                            "kwargs_model": kwargs_container,
                            "parsed_parameter": parameter,
                        },
                        default=kwargs_container.default if kwargs_container.default is not Empty else attr.NOTHING,
                        validator=_create_validators(annotation=annotation, kwargs_model=kwargs_container),
                    )
            elif parameter.has_default:
                attribute = attr.attrib(type=annotation, default=parameter.default)
            else:
                attribute = attr.attrib(
                    type=annotation, default=None if parameter.parsed_type.is_optional else attr.NOTHING
                )

            attributes[parameter.name] = attribute

        model: type[AttrsSignatureModel] = attrs.make_class(
            f"{fn_name}_signature_model",
            attrs=attributes,
            bases=(AttrsSignatureModel,),
            slots=True,
            kw_only=True,
        )
        model.return_annotation = parsed_signature.return_type.annotation  # pyright: ignore
        model.dependency_name_set = dependency_names  # pyright: ignore
        model.populate_signature_fields()  # pyright: ignore
        return model



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, cast

from sqlalchemy.ext.asyncio import AsyncConnection, AsyncEngine, AsyncSession, async_sessionmaker, create_async_engine

from litestar.types import Empty
from litestar.utils import (
    delete_litestar_scope_state,
    get_litestar_scope_state,
)

from .common import SESSION_SCOPE_KEY, SESSION_TERMINUS_ASGI_EVENTS, GenericSessionConfig, GenericSQLAlchemyConfig

if TYPE_CHECKING:
    from typing import Any, Callable

    from sqlalchemy.orm import Session

    from litestar.datastructures.state import State
    from litestar.types import BeforeMessageSendHookHandler, EmptyType, Message, Scope

__all__ = ("SQLAlchemyAsyncConfig", "AsyncSessionConfig")


async def default_before_send_handler(message: Message, _: State, scope: Scope) -> None:
    """Handle closing and cleaning up sessions before sending.

    Args:
        message: ASGI-``Message``
        _: A ``State`` (not used)
        scope: An ASGI-``Scope``

    Returns:
        None
    """
    session = cast("AsyncSession | None", get_litestar_scope_state(scope, SESSION_SCOPE_KEY))
    if session and message["type"] in SESSION_TERMINUS_ASGI_EVENTS:
        await session.close()
        delete_litestar_scope_state(scope, SESSION_SCOPE_KEY)


@dataclass
class AsyncSessionConfig(GenericSessionConfig[AsyncConnection, AsyncEngine, AsyncSession]):
    """SQLAlchemy async session config."""

    sync_session_class: type[Session] | None | EmptyType = Empty
    """A :class:`Session <sqlalchemy.orm.Session>` subclass or other callable which will be used to construct the
    :class:`Session <sqlalchemy.orm.Session>` which will be proxied. This parameter may be used to provide custom
    :class:`Session <sqlalchemy.orm.Session>` subclasses. Defaults to the
    :attr:`AsyncSession.sync_session_class <sqlalchemy.ext.asyncio.AsyncSession.sync_session_class>` class-level
    attribute."""


@dataclass
class SQLAlchemyAsyncConfig(GenericSQLAlchemyConfig[AsyncEngine, AsyncSession, async_sessionmaker]):
    """Async SQLAlchemy Configuration."""

    create_engine_callable: Callable[[str], AsyncEngine] = create_async_engine
    """Callable that creates an :class:`AsyncEngine <sqlalchemy.ext.asyncio.AsyncEngine>` instance or instance of its
    subclass.
    """
    session_config: AsyncSessionConfig = field(default_factory=AsyncSessionConfig)
    """Configuration options for the :class:`async_sessionmaker<sqlalchemy.ext.asyncio.async_sessionmaker>`."""
    session_maker_class: type[async_sessionmaker] = async_sessionmaker
    """Sessionmaker class to use."""
    before_send_handler: BeforeMessageSendHookHandler = default_before_send_handler
    """Handler to call before the ASGI message is sent.

    The handler should handle closing the session stored in the ASGI scope, if it's still open, and committing and
    uncommitted data.
    """

    @property
    def signature_namespace(self) -> dict[str, Any]:
        """Return the plugin's signature namespace.

        Returns:
            A string keyed dict of names to be added to the namespace for signature forward reference resolution.
        """
        return {"AsyncEngine": AsyncEngine, "AsyncSession": AsyncSession}

    async def on_shutdown(self, state: State) -> None:
        """Disposes of the SQLAlchemy engine.

        Args:
            state: The ``Litestar.state`` instance.

        Returns:
            None
        """
        engine = cast("AsyncEngine", state.pop(self.engine_app_state_key))
        await engine.dispose()



from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Any, Literal

__all__ = (
    "TypeScriptAnonymousInterface",
    "TypeScriptArray",
    "TypeScriptConst",
    "TypeScriptContainer",
    "TypeScriptElement",
    "TypeScriptEnum",
    "TypeScriptInterface",
    "TypeScriptIntersection",
    "TypeScriptLiteral",
    "TypeScriptNamespace",
    "TypeScriptPrimitive",
    "TypeScriptProperty",
    "TypeScriptType",
    "TypeScriptUnion",
)


def _as_string(value: Any) -> str:
    if isinstance(value, str):
        return '"' + value + '"'

    if isinstance(value, bool):
        return "true" if value else "false"

    if value is None:
        return "null"

    return str(value)


class TypeScriptElement(ABC):
    """A class representing a TypeScript type element."""

    @abstractmethod
    def write(self) -> str:
        """Write a typescript value corresponding to the given typescript element.

        Returns:
            A typescript string
        """
        raise NotImplementedError("")


class TypeScriptContainer(TypeScriptElement):
    """A class representing a TypeScript type container."""

    name: str

    @abstractmethod
    def write(self) -> str:
        """Write a typescript value corresponding to the given typescript container.

        Returns:
            A typescript string
        """
        raise NotImplementedError("")


@dataclass(unsafe_hash=True)
class TypeScriptIntersection(TypeScriptElement):
    """A class representing a TypeScript intersection type."""

    types: tuple[TypeScriptElement, ...]

    def write(self) -> str:
        """Write a typescript intersection value.

        Example:
            { prop: string } & { another: number }

        Returns:
            A typescript string
        """
        return " & ".join(t.write() for t in self.types)


@dataclass(unsafe_hash=True)
class TypeScriptUnion(TypeScriptElement):
    """A class representing a TypeScript union type."""

    types: tuple[TypeScriptElement, ...]

    def write(self) -> str:
        """Write a typescript union value.

        Example:
            string | number

        Returns:
            A typescript string
        """
        return " | ".join(sorted(t.write() for t in self.types))


@dataclass(unsafe_hash=True)
class TypeScriptPrimitive(TypeScriptElement):
    """A class representing a TypeScript primitive type."""

    type: Literal[
        "string", "number", "boolean", "any", "null", "undefined", "symbol", "Record<string, unknown>", "unknown[]"
    ]

    def write(self) -> str:
        """Write a typescript primitive type.

        Example:
            null

        Returns:
            A typescript string
        """
        return self.type


@dataclass(unsafe_hash=True)
class TypeScriptLiteral(TypeScriptElement):
    """A class representing a TypeScript literal type."""

    value: str | int | float | bool | None

    def write(self) -> str:
        """Write a typescript literal type.

        Example:
            "someValue"

        Returns:
            A typescript string
        """
        return _as_string(self.value)


@dataclass(unsafe_hash=True)
class TypeScriptArray(TypeScriptElement):
    """A class representing a TypeScript array type."""

    item_type: TypeScriptElement

    def write(self) -> str:
        """Write a typescript array type.

        Example:
            number[]

        Returns:
            A typescript string
        """
        value = (
            f"({self.item_type.write()})"
            if isinstance(self.item_type, (TypeScriptUnion, TypeScriptIntersection))
            else self.item_type.write()
        )
        return f"{value}[]"


@dataclass(unsafe_hash=True)
class TypeScriptProperty(TypeScriptElement):
    """A class representing a TypeScript interface property."""

    required: bool
    key: str
    value: TypeScriptElement

    def write(self) -> str:
        """Write a typescript property. This class is used exclusively inside interfaces.

        Example:
            key: string;
            optional?: number;

        Returns:
            A typescript string
        """
        return f"{self.key}{':' if self.required else '?:'} {self.value.write()};"


@dataclass(unsafe_hash=True)
class TypeScriptAnonymousInterface(TypeScriptElement):
    """A class representing a TypeScript anonymous interface."""

    properties: tuple[TypeScriptProperty, ...]

    def write(self) -> str:
        """Write a typescript interface object, without a name.

        Example:
            {
                key: string;
                optional?: number;
            }

        Returns:
            A typescript string
        """
        props = "\t" + "\n\t".join([prop.write() for prop in sorted(self.properties, key=lambda prop: prop.key)])
        return f"{{\n{props}\n}}"


@dataclass(unsafe_hash=True)
class TypeScriptInterface(TypeScriptContainer):
    """A class representing a TypeScript interface."""

    name: str
    properties: tuple[TypeScriptProperty, ...]

    def write(self) -> str:
        """Write a typescript interface.

        Example:
            export interface MyInterface {
                key: string;
                optional?: number;
            };

        Returns:
            A typescript string
        """
        interface = TypeScriptAnonymousInterface(properties=self.properties)
        return f"export interface {self.name} {interface.write()};"


@dataclass(unsafe_hash=True)
class TypeScriptEnum(TypeScriptContainer):
    """A class representing a TypeScript enum."""

    name: str
    values: tuple[tuple[str, str], ...] | tuple[tuple[str, int | float], ...]

    def write(self) -> str:
        """Write a typescript enum.

        Example:
            export enum MyEnum {
                DOG = "canine",
                CAT = "feline",
            };

        Returns:
            A typescript string
        """
        members = "\t" + "\n\t".join(
            [f"{key} = {_as_string(value)}," for key, value in sorted(self.values, key=lambda member: member[0])]
        )
        return f"export enum {self.name} {{\n{members}\n}};"


@dataclass(unsafe_hash=True)
class TypeScriptType(TypeScriptContainer):
    """A class representing a TypeScript type."""

    name: str
    value: TypeScriptElement

    def write(self) -> str:
        """Write a typescript type.

        Example:
            export type MyType = number | "42";

        Returns:
            A typescript string
        """
        return f"export type {self.name} = {self.value.write()};"


@dataclass(unsafe_hash=True)
class TypeScriptConst(TypeScriptContainer):
    """A class representing a TypeScript const."""

    name: str
    value: TypeScriptPrimitive | TypeScriptLiteral

    def write(self) -> str:
        """Write a typescript const.

        Example:
            export const MyConst: number;

        Returns:
            A typescript string
        """
        return f"export const {self.name}: {self.value.write()};"


@dataclass(unsafe_hash=True)
class TypeScriptNamespace(TypeScriptContainer):
    """A class representing a TypeScript namespace."""

    name: str
    values: tuple[TypeScriptContainer, ...]

    def write(self) -> str:
        """Write a typescript namespace.

        Example:
            export MyNamespace {
                export const MyConst: number;
            }

        Returns:
            A typescript string
        """
        members = "\t" + "\n\n\t".join([value.write() for value in sorted(self.values, key=lambda el: el.name)])
        return f"export namespace {self.name} {{\n{members}\n}};"



"""DTO backends do the heavy lifting of decoding and validating raw bytes into domain models, and
back again, to bytes.
"""
from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Generic, TypeVar

from litestar._openapi.schema_generation import create_schema
from litestar._signature.field import SignatureField

from .utils import build_annotation_for_backend

if TYPE_CHECKING:
    from typing import Any, Final

    from litestar.dto.factory.types import FieldDefinitionsType
    from litestar.dto.interface import ConnectionContext
    from litestar.openapi.spec import Reference, Schema
    from litestar.types.serialization import LitestarEncodableType
    from litestar.utils.signature import ParsedType

__all__ = ("AbstractDTOBackend", "BackendContext")

BackendT = TypeVar("BackendT")


class BackendContext:
    """Context required by DTO backends to perform their work."""

    __slots__ = ("parsed_type", "field_definitions", "model_type", "reverse_name_map")

    def __init__(self, parsed_type: ParsedType, field_definitions: FieldDefinitionsType, model_type: type[Any]) -> None:
        """Create a backend context.

        Args:
            parsed_type: Parsed type.
            field_definitions: Field definitions.
            model_type: Model type.
        """
        self.parsed_type: Final[ParsedType] = parsed_type
        self.field_definitions: Final[FieldDefinitionsType] = field_definitions
        self.model_type: Final[type[Any]] = model_type
        self.reverse_name_map = {
            f.serialization_name: f.name for f in field_definitions.values() if f.serialization_name
        }


class AbstractDTOBackend(ABC, Generic[BackendT]):
    __slots__ = (
        "data_container_type",
        "annotation",
        "context",
    )

    def __init__(self, context: BackendContext) -> None:
        """Create dto backend instance.

        Args:
            context: context of the type represented by this backend.
        """
        self.context = context
        self.data_container_type = self.create_data_container_type(context)
        self.annotation = build_annotation_for_backend(context.parsed_type.annotation, self.data_container_type)

    @abstractmethod
    def create_data_container_type(self, context: BackendContext) -> type[BackendT]:
        """Create a data container type to represent the context type.

        Args:
            context: Context of the type to create a data container for.

        Returns:
            A ``BackendT`` class.
        """

    @abstractmethod
    def parse_raw(self, raw: bytes, connection_context: ConnectionContext) -> Any:
        """Parse raw bytes into primitive python types.

        Args:
            raw: bytes
            connection_context: Information about the active connection.

        Returns:
            The raw bytes parsed into primitive python types.
        """

    @abstractmethod
    def populate_data_from_builtins(self, data: Any) -> Any:
        """Populate model instance from builtin types.

        Args:
            model_type: Type of model to populate.
            data: Builtin type.

        Returns:
            Instance or collection of ``model_type`` instances.
        """

    @abstractmethod
    def populate_data_from_raw(self, raw: bytes, connection_context: ConnectionContext) -> Any:
        """Parse raw bytes into instance of `model_type`.

        Args:
            raw: bytes
            connection_context: Information about the active connection.

        Returns:
            Instance or collection of ``model_type`` instances.
        """

    @abstractmethod
    def encode_data(self, data: Any, connection_context: ConnectionContext) -> LitestarEncodableType:
        """Encode data into a ``LitestarEncodableType``.

        Args:
            data: Data to encode.
            connection_context: Information about the active connection.

        Returns:
            Encoded data.
        """

    def create_openapi_schema(self, generate_examples: bool, schemas: dict[str, Schema]) -> Reference | Schema:
        """Create a RequestBody model for the given RouteHandler or return None."""
        field = SignatureField.create(self.annotation)
        return create_schema(field=field, generate_examples=generate_examples, plugins=[], schemas=schemas)



from __future__ import annotations

import re
from copy import copy
from dataclasses import asdict
from http import HTTPStatus
from inspect import Signature
from operator import attrgetter
from typing import TYPE_CHECKING, Any, Iterator

from typing_extensions import get_args, get_origin

from litestar._openapi.schema_generation import create_schema
from litestar._signature.field import SignatureField
from litestar.enums import MediaType
from litestar.exceptions import HTTPException, ValidationException
from litestar.openapi.spec import OpenAPIResponse
from litestar.openapi.spec.enums import OpenAPIFormat, OpenAPIType
from litestar.openapi.spec.header import OpenAPIHeader
from litestar.openapi.spec.media_type import OpenAPIMediaType
from litestar.openapi.spec.schema import Schema
from litestar.response import Response as LitestarResponse
from litestar.response_containers import File, Redirect, Stream, Template
from litestar.types.builtin_types import NoneType
from litestar.utils import get_enum_string_value, get_name, is_class_and_subclass

if TYPE_CHECKING:
    from litestar.datastructures.cookie import Cookie
    from litestar.handlers.http_handlers import HTTPRouteHandler
    from litestar.openapi.spec.responses import Responses
    from litestar.plugins import OpenAPISchemaPluginProtocol


__all__ = (
    "create_additional_responses",
    "create_cookie_schema",
    "create_error_responses",
    "create_responses",
    "create_success_response",
)

CAPITAL_LETTERS_PATTERN = re.compile(r"(?=[A-Z])")


def pascal_case_to_text(string: str) -> str:
    """Given a 'PascalCased' string, return its split form- 'Pascal Cased'."""
    return " ".join(re.split(CAPITAL_LETTERS_PATTERN, string)).strip()


def create_cookie_schema(cookie: Cookie) -> Schema:
    """Given a Cookie instance, return its corresponding OpenAPI schema.

    Args:
        cookie: Cookie

    Returns:
        Schema
    """
    cookie_copy = copy(cookie)
    cookie_copy.value = "<string>"
    value = cookie_copy.to_header(header="")
    return Schema(description=cookie.description or "", example=value)


def create_success_response(  # noqa: C901
    route_handler: HTTPRouteHandler,
    generate_examples: bool,
    plugins: list[OpenAPISchemaPluginProtocol],
    schemas: dict[str, Schema],
) -> OpenAPIResponse:
    """Create the schema for a success response."""
    return_annotation = route_handler.parsed_fn_signature.return_type.annotation
    default_descriptions: dict[Any, str] = {
        Stream: "Stream Response",
        Redirect: "Redirect Response",
        File: "File Download",
    }
    description = (
        route_handler.response_description
        or default_descriptions.get(return_annotation)
        or HTTPStatus(route_handler.status_code).description
    )

    if return_annotation not in {Signature.empty, None, NoneType, Redirect, File, Stream}:
        if return_annotation is Template:
            return_annotation = str
            route_handler.media_type = get_enum_string_value(MediaType.HTML)
        elif is_class_and_subclass(get_origin(return_annotation), LitestarResponse):
            return_annotation = get_args(return_annotation)[0] or Any

        if dto := route_handler.resolve_return_dto():
            result = dto.create_openapi_schema("return", str(route_handler), generate_examples, schemas)
        else:
            result = create_schema(
                field=SignatureField.create(field_type=return_annotation),
                generate_examples=generate_examples,
                plugins=plugins,
                schemas=schemas,
            )

        schema = result if isinstance(result, Schema) else schemas[result.value]

        schema.content_encoding = route_handler.content_encoding
        schema.content_media_type = route_handler.content_media_type

        response = OpenAPIResponse(
            content={
                route_handler.media_type: OpenAPIMediaType(
                    schema=result,
                )
            },
            description=description,
        )

    elif return_annotation is Redirect:
        response = OpenAPIResponse(
            content=None,
            description=description,
            headers={
                "location": OpenAPIHeader(
                    schema=Schema(type=OpenAPIType.STRING), description="target path for the redirect"
                )
            },
        )

    elif return_annotation in (File, Stream):
        response = OpenAPIResponse(
            content={
                route_handler.media_type: OpenAPIMediaType(
                    schema=Schema(
                        type=OpenAPIType.STRING,
                        content_encoding=route_handler.content_encoding or "application/octet-stream",
                        content_media_type=route_handler.content_media_type,
                    ),
                )
            },
            description=description,
            headers={
                "content-length": OpenAPIHeader(
                    schema=Schema(type=OpenAPIType.STRING), description="File size in bytes"
                ),
                "last-modified": OpenAPIHeader(
                    schema=Schema(type=OpenAPIType.STRING, format=OpenAPIFormat.DATE_TIME),
                    description="Last modified data-time in RFC 2822 format",
                ),
                "etag": OpenAPIHeader(schema=Schema(type=OpenAPIType.STRING), description="Entity tag"),
            },
        )

    else:
        response = OpenAPIResponse(
            content=None,
            description=description,
        )

    if response.headers is None:
        response.headers = {}

    for response_header in route_handler.resolve_response_headers():
        header = OpenAPIHeader()
        for attribute_name, attribute_value in ((k, v) for k, v in asdict(response_header).items() if v is not None):
            if attribute_name == "value":
                header.schema = create_schema(
                    field=SignatureField.create(field_type=type(attribute_value)),
                    generate_examples=False,
                    plugins=plugins,
                    schemas=schemas,
                )

            elif attribute_name != "documentation_only":
                setattr(header, attribute_name, attribute_value)

        response.headers[response_header.name] = header

    if cookies := route_handler.resolve_response_cookies():
        response.headers["Set-Cookie"] = OpenAPIHeader(
            schema=Schema(
                all_of=[create_cookie_schema(cookie=cookie) for cookie in sorted(cookies, key=attrgetter("key"))]
            )
        )

    return response


def create_error_responses(exceptions: list[type[HTTPException]]) -> Iterator[tuple[str, OpenAPIResponse]]:
    """Create the schema for error responses, if any."""
    grouped_exceptions: dict[int, list[type[HTTPException]]] = {}
    for exc in exceptions:
        if not grouped_exceptions.get(exc.status_code):
            grouped_exceptions[exc.status_code] = []
        grouped_exceptions[exc.status_code].append(exc)
    for status_code, exception_group in grouped_exceptions.items():
        exceptions_schemas = [
            Schema(
                type=OpenAPIType.OBJECT,
                required=["detail", "status_code"],
                properties={
                    "status_code": Schema(type=OpenAPIType.INTEGER),
                    "detail": Schema(type=OpenAPIType.STRING),
                    "extra": Schema(
                        type=[OpenAPIType.NULL, OpenAPIType.OBJECT, OpenAPIType.ARRAY], additional_properties=Schema()
                    ),
                },
                description=pascal_case_to_text(get_name(exc)),
                examples=[{"status_code": status_code, "detail": HTTPStatus(status_code).phrase, "extra": {}}],
            )
            for exc in exception_group
        ]
        if len(exceptions_schemas) > 1:  # noqa: SIM108
            schema = Schema(one_of=exceptions_schemas)
        else:
            schema = exceptions_schemas[0]
        yield str(status_code), OpenAPIResponse(
            description=HTTPStatus(status_code).description,
            content={MediaType.JSON: OpenAPIMediaType(schema=schema)},
        )


def create_additional_responses(
    route_handler: HTTPRouteHandler,
    plugins: list[OpenAPISchemaPluginProtocol],
    schemas: dict[str, Schema],
) -> Iterator[tuple[str, OpenAPIResponse]]:
    """Create the schema for additional responses, if any."""
    if not route_handler.responses:
        return

    for status_code, additional_response in route_handler.responses.items():
        schema = create_schema(
            field=SignatureField.create(field_type=additional_response.data_container),
            generate_examples=additional_response.generate_examples,
            plugins=plugins,
            schemas=schemas,
        )
        yield str(status_code), OpenAPIResponse(
            description=additional_response.description,
            content={additional_response.media_type: OpenAPIMediaType(schema=schema)},
        )


def create_responses(
    route_handler: HTTPRouteHandler,
    raises_validation_error: bool,
    generate_examples: bool,
    plugins: list[OpenAPISchemaPluginProtocol],
    schemas: dict[str, Schema],
) -> Responses | None:
    """Create a Response model embedded in a `Responses` dictionary for the given RouteHandler or return None."""

    responses: Responses = {
        str(route_handler.status_code): create_success_response(
            generate_examples=generate_examples, plugins=plugins, route_handler=route_handler, schemas=schemas
        ),
    }

    exceptions = list(route_handler.raises or [])
    if raises_validation_error and ValidationException not in exceptions:
        exceptions.append(ValidationException)
    for status_code, response in create_error_responses(exceptions=exceptions):
        responses[status_code] = response

    for status_code, response in create_additional_responses(
        route_handler=route_handler, plugins=plugins, schemas=schemas
    ):
        responses[status_code] = response

    return responses or None



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections import deque
    from collections.abc import Collection
    from datetime import date, datetime, time
    from decimal import Decimal
    from enum import Enum, IntEnum
    from ipaddress import (
        IPv4Address,
        IPv4Interface,
        IPv4Network,
        IPv6Address,
        IPv6Interface,
        IPv6Network,
    )
    from pathlib import Path, PurePath
    from re import Pattern
    from uuid import UUID

    from msgspec import Raw, Struct
    from msgspec.msgpack import Ext
    from pydantic import (
        BaseModel,
        ByteSize,
        ConstrainedBytes,
        ConstrainedDate,
        NameEmail,
        SecretField,
        StrictBool,
    )
    from pydantic.color import Color
    from typing_extensions import TypeAlias

    from litestar.types import DataclassProtocol

__all__ = ("LitestarEncodableType",)

EncodableBuiltinType: TypeAlias = "None | bool | int | float | str | bytes | bytearray"
EncodableBuiltinCollectionType: TypeAlias = "list | tuple | set | frozenset | dict | Collection"
EncodableStdLibType: TypeAlias = (
    "date | datetime | deque | time | UUID | Decimal | Enum | IntEnum | DataclassProtocol | Path | PurePath | Pattern"
)
EncodableStdLibIPType: TypeAlias = (
    "IPv4Address | IPv4Interface | IPv4Network | IPv6Address | IPv6Interface | IPv6Network"
)
EncodableMsgSpecType: TypeAlias = "Ext | Raw | Struct"
EncodablePydanticType: TypeAlias = (
    "BaseModel | ByteSize | ConstrainedBytes | ConstrainedDate | NameEmail | SecretField | StrictBool | Color"
)

LitestarEncodableType: TypeAlias = "EncodableBuiltinType | EncodableBuiltinCollectionType | EncodableStdLibType | EncodableStdLibIPType | EncodableMsgSpecType | EncodablePydanticType"



from __future__ import annotations

from click import Context, group, option, pass_context

from ._utils import LitestarEnv, LitestarExtensionGroup
from .commands import core, schema, sessions

__all__ = ("litestar_group",)


@group(cls=LitestarExtensionGroup)
@option("--app", "app_path", help="Module path to a Litestar application")
@pass_context
def litestar_group(ctx: Context, app_path: str | None) -> None:
    """Litestar CLI."""

    ctx.obj = lambda: LitestarEnv.from_env(app_path)


# add sub commands here

litestar_group.add_command(core.info_command)
litestar_group.add_command(core.run_command)
litestar_group.add_command(core.routes_command)
litestar_group.add_command(core.version_command)
litestar_group.add_command(sessions.sessions_group)
litestar_group.add_command(schema.schema_group)



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

from typing_extensions import get_origin

if TYPE_CHECKING:
    from typing import Any, Iterable

__all__ = ("build_annotation_for_backend",)

T = TypeVar("T")


def build_annotation_for_backend(annotation: Any, model: type[T]) -> type[T] | type[Iterable[T]]:
    """A helper to re-build a generic outer type with new inner type.

    Args:
        annotation: The original annotation on the handler signature
        model: The data container type

    Returns:
        Annotation with new inner type if applicable.
    """
    origin = get_origin(annotation)
    if not origin:
        return model
    try:
        return origin[model]  # type:ignore[no-any-return]
    except TypeError:  # pragma: no cover
        return annotation.copy_with((model,))  # type:ignore[no-any-return]



from __future__ import annotations

from inspect import isclass
from typing import TYPE_CHECKING, Any

from litestar.exceptions import ImproperlyConfiguredException
from litestar.types import Empty
from litestar.utils import Ref, is_async_callable

__all__ = ("Provide",)


if TYPE_CHECKING:
    from litestar._signature import SignatureModel
    from litestar.types import AnyCallable


class Provide:
    """Wrapper class for dependency injection"""

    __slots__ = (
        "dependency",
        "has_sync_callable",
        "has_class_dependency",
        "signature_model",
        "sync_to_thread",
        "use_cache",
        "value",
    )

    signature_model: type[SignatureModel]
    dependency: Ref[AnyCallable]

    def __init__(
        self,
        dependency: AnyCallable | type,
        use_cache: bool = False,
        sync_to_thread: bool = False,
    ) -> None:
        """Initialize ``Provide``

        Args:
            dependency: Callable to call or class to instantiate. The result is then injected as a dependency.
            use_cache: Cache the dependency return value. Defaults to False.
            sync_to_thread: Run sync code in an async thread. Defaults to False.
        """
        if not callable(dependency):
            raise ImproperlyConfiguredException("Provider dependency must a callable value")

        self.dependency = Ref["AnyCallable"](dependency)
        self.has_sync_callable = isclass(dependency) or not is_async_callable(dependency)
        self.sync_to_thread = sync_to_thread
        self.use_cache = use_cache
        self.value: Any = Empty

    async def __call__(self, **kwargs: Any) -> Any:
        """Call the provider's dependency."""

        if self.use_cache and self.value is not Empty:
            return self.value

        if self.has_sync_callable:
            value = self.dependency.value(**kwargs)
        else:
            value = await self.dependency.value(**kwargs)

        if self.use_cache:
            self.value = value

        return value

    def __eq__(self, other: Any) -> bool:
        # check if memory address is identical, otherwise compare attributes
        return other is self or (
            isinstance(other, self.__class__)
            and other.dependency == self.dependency
            and other.use_cache == self.use_cache
            and other.value == self.value
        )



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import TYPE_CHECKING, Generic, TypeVar

from litestar.dto.factory.backends import MsgspecDTOBackend, PydanticDTOBackend
from litestar.dto.factory.backends.abc import BackendContext
from litestar.dto.interface import ConnectionContext, DTOInterface
from litestar.enums import RequestEncodingType
from litestar.types.builtin_types import NoneType
from litestar.utils.signature import ParsedType

from .config import DTOConfig
from .exc import InvalidAnnotation
from .field import Mark
from .types import FieldDefinition, FieldDefinitionsType, NestedFieldDefinition
from .utils import parse_configs_from_annotation

if TYPE_CHECKING:
    from typing import Any, ClassVar, Collection, Generator

    from typing_extensions import Self

    from litestar.dto.interface import HandlerContext
    from litestar.dto.types import ForType
    from litestar.openapi.spec import Reference, Schema
    from litestar.types.serialization import LitestarEncodableType

    from .backends import AbstractDTOBackend

__all__ = ["AbstractDTOFactory"]

DataT = TypeVar("DataT")


class AbstractDTOFactory(DTOInterface, Generic[DataT], metaclass=ABCMeta):
    """Base class for DTO types."""

    __slots__ = ("connection_context",)

    config: ClassVar[DTOConfig]
    """Config objects to define properties of the DTO."""
    model_type: ClassVar[type[Any]]
    """If ``annotation`` is an iterable, this is the inner type, otherwise will be the same as ``annotation``."""

    _reverse_field_mappings: ClassVar[dict[str, FieldDefinition]]
    _type_backend_map: ClassVar[dict[tuple[ForType, ParsedType, RequestEncodingType | str | None], AbstractDTOBackend]]
    _handler_backend_map: ClassVar[dict[tuple[ForType, str], AbstractDTOBackend]]

    def __init__(self, connection_context: ConnectionContext) -> None:
        """Create an AbstractDTOFactory type.

        Args:
            connection_context: A :class:`ConnectionContext <.ConnectionContext>` instance, which provides
                information about the connection.
        """
        self.connection_context = connection_context

    def __class_getitem__(cls, annotation: Any) -> type[Self]:
        parsed_type = ParsedType(annotation)

        if (parsed_type.is_optional and len(parsed_type.args) > 2) or (
            parsed_type.is_union and not parsed_type.is_optional
        ):
            raise InvalidAnnotation(
                "Unions are currently not supported as type argument to DTO. Want this? Open an issue."
            )

        if parsed_type.is_forward_ref:
            raise InvalidAnnotation("Forward references are not supported as type argument to DTO")

        # if a configuration is not provided, and the type narrowing is a type var, we don't want to create a subclass
        configs = parse_configs_from_annotation(parsed_type)
        if parsed_type.is_type_var and not configs:
            return cls

        if configs:
            # provided config is always preferred
            config = configs[0]
        elif hasattr(cls, "config"):
            # if no config is provided, but the class has one assigned, use that
            config = cls.config
        else:
            # otherwise, create a new config
            config = DTOConfig()

        cls_dict: dict[str, Any] = {
            "config": config,
            "_reverse_field_mappings": {},
            "_type_backend_map": {},
            "_handler_backend_map": {},
        }
        if not parsed_type.is_type_var:
            cls_dict.update(model_type=parsed_type.annotation)

        return type(f"{cls.__name__}[{annotation}]", (cls,), cls_dict)

    def builtins_to_data_type(self, builtins: Any) -> Any:
        """Coerce the unstructured data into the data type."""
        backend = self._handler_backend_map[("data", self.connection_context.handler_id)]
        return backend.populate_data_from_builtins(builtins)

    def bytes_to_data_type(self, raw: bytes) -> Any:
        """Return the data held by the DTO."""
        backend = self._handler_backend_map[("data", self.connection_context.handler_id)]
        return backend.populate_data_from_raw(raw, self.connection_context)

    def data_to_encodable_type(self, data: DataT | Collection[DataT]) -> LitestarEncodableType:
        backend = self._handler_backend_map[("return", self.connection_context.handler_id)]
        return backend.encode_data(data, self.connection_context)

    @classmethod
    @abstractmethod
    def generate_field_definitions(cls, model_type: type[Any]) -> Generator[FieldDefinition, None, None]:
        """Generate ``FieldDefinition`` instances from ``model_type``.

        Yields:
            ``FieldDefinition`` instances.
        """

    @classmethod
    @abstractmethod
    def detect_nested_field(cls, field_definition: FieldDefinition) -> bool:
        """Return ``True`` if ``field_definition`` represents a nested model field.

        Args:
            field_definition: inspect type to determine if field definition represents a nested model.

        Returns:
            ``True`` if ``field_definition`` represents a nested model field.
        """

    @classmethod
    def on_registration(cls, handler_context: HandlerContext) -> None:
        """Called each time the DTO type is encountered during signature modelling.

        Args:
            handler_context: A :class:`HandlerContext <.HandlerContext>` instance. Provides information about the
                handler and application of the DTO.
        """

        if handler_context.parsed_type.is_collection:
            if len(handler_context.parsed_type.inner_types) != 1:
                raise InvalidAnnotation("AbstractDTOFactory only supports homogeneous collection types")
            handler_type = handler_context.parsed_type.inner_types[0]
        else:
            handler_type = handler_context.parsed_type

        if not handler_type.is_subclass_of(cls.model_type):
            raise InvalidAnnotation(
                f"DTO narrowed with '{cls.model_type}', handler type is '{handler_context.parsed_type.annotation}'"
            )

        key = (handler_context.dto_for, handler_context.parsed_type, handler_context.request_encoding_type)
        backend = cls._type_backend_map.get(key)
        if backend is None:
            backend_type: type[AbstractDTOBackend]
            if handler_context.request_encoding_type in {
                RequestEncodingType.URL_ENCODED,
                RequestEncodingType.MULTI_PART,
            }:
                backend_type = PydanticDTOBackend
            else:
                backend_type = MsgspecDTOBackend

            backend_context = BackendContext(
                handler_context.parsed_type,
                _parse_model(cls, handler_type.annotation, cls.config, handler_context.dto_for),
                handler_type.annotation,
            )
            backend = cls._type_backend_map.setdefault(key, backend_type(backend_context))
        cls._handler_backend_map[(handler_context.dto_for, handler_context.handler_id)] = backend

    @classmethod
    def create_openapi_schema(
        cls,
        dto_for: ForType,
        handler_id: str,
        generate_examples: bool,
        schemas: dict[str, Schema],
    ) -> Reference | Schema:
        """Create an OpenAPI request body.

        Returns:
            OpenAPI request body.
        """
        backend = cls._handler_backend_map[(dto_for, handler_id)]
        return backend.create_openapi_schema(generate_examples, schemas)


def _parse_model(
    dto_factory_type: type[AbstractDTOFactory],
    model_type: Any,
    config: DTOConfig,
    dto_for: ForType,
    nested_depth: int = 0,
) -> FieldDefinitionsType:
    """Reduce :attr:`model_type` to :class:`FieldDefinitionsType`.

    .. important::
        Implementations must respect the :attr:`config` object. For example:
            - fields marked private must never be included in the field definitions.
            - if a ``purpose`` is declared, then read-only fields must be taken into account.
            - field mappings must be implemented.
            - additional fields must be included, subject to ``purpose``.
            - nested depth and nested recursion depth must be adhered to.

    Returns:
        Fields for data transfer.

        Key is the name of the new field, and value is a tuple of type and default value pairs.

        Add a new field called "new_field", that is a string, and required:
        {"new_field": (str, ...)}

        Add a new field called "new_field", that is a string, and not-required:
        {"new_field": (str, "default")}

        Add a new field called "new_field", that may be `None`:
        {"new_field": (str | None, None)}
    """
    defined_fields: dict[str, FieldDefinition | NestedFieldDefinition] = {}
    for field_definition in dto_factory_type.generate_field_definitions(model_type):
        if _should_exclude_field(field_definition, config, dto_for):
            continue

        if rename := config.rename_fields.get(field_definition.name):
            field_definition = field_definition.copy_with(serialization_name=rename)  # noqa: PLW2901

        if dto_factory_type.detect_nested_field(field_definition):
            nested_field_definition = _handle_nested(dto_factory_type, field_definition, nested_depth, config, dto_for)
            if nested_field_definition is not None:
                defined_fields[field_definition.name] = nested_field_definition
            continue

        defined_fields[field_definition.name] = field_definition
    return defined_fields


def _should_exclude_field(field_definition: FieldDefinition, config: DTOConfig, dto_for: ForType) -> bool:
    """Returns ``True`` where a field should be excluded from data transfer.

    Args:
        field_definition: defined DTO field
        config: DTO configuration
        dto_for: indicates whether the DTO is for the request body or response.

    Returns:
        ``True`` if the field should not be included in any data transfer.
    """
    field_name = field_definition.name
    dto_field = field_definition.dto_field
    excluded = field_name in config.exclude
    private = dto_field and dto_field.mark is Mark.PRIVATE
    read_only_for_write = dto_for == "data" and dto_field and dto_field.mark is Mark.READ_ONLY
    return bool(excluded or private or read_only_for_write)


def _handle_nested(
    dto_factory_type: type[AbstractDTOFactory],
    field_definition: FieldDefinition,
    nested_depth: int,
    config: DTOConfig,
    dto_for: ForType,
) -> NestedFieldDefinition | None:
    if nested_depth == config.max_nested_depth:
        return None

    nested = NestedFieldDefinition(
        field_definition=field_definition,
        nested_type=_get_model_type(field_definition.annotation),
    )

    nested.nested_field_definitions = _parse_model(
        dto_factory_type, nested.nested_type, config, dto_for, nested_depth + 1
    )
    return nested


def _get_model_type(annotation: type) -> Any:
    """Get model type represented by the DTO.

    If ``annotation`` is a collection, then the inner type is returned.

    Args:
        annotation: any type.

    Returns:
        The model type that is represented by the DTO.
    """
    parsed_type = ParsedType(annotation)
    if parsed_type.is_collection:
        return parsed_type.inner_types[0].annotation
    if parsed_type.is_optional:
        return next(t for t in parsed_type.inner_types if t.annotation is not NoneType).annotation
    return parsed_type.annotation



from functools import partial
from os import PathLike
from pathlib import Path
from typing import (
    TYPE_CHECKING,
    Any,
    AsyncIterable,
    AsyncIterator,
    Callable,
    Dict,
    Iterable,
    Iterator,
    Literal,
    Mapping,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from litestar.enums import ScopeType

from .asgi_types import ASGIApp
from .callable_types import ExceptionHandler

if TYPE_CHECKING:
    from litestar.datastructures.cookie import Cookie
    from litestar.datastructures.response_header import ResponseHeader
    from litestar.di import Provide
    from litestar.middleware.base import DefineMiddleware, MiddlewareProtocol
    from litestar.params import ParameterKwarg
else:
    BaseHTTPMiddleware = Any
    Cookie = Any
    DefineMiddleware = Any
    ImmutableState = Any
    MiddlewareProtocol = Any
    ParameterKwarg = Any
    Provide = Any
    ResponseHeader = Any

T = TypeVar("T")


Dependencies = Mapping[str, Provide]
ExceptionHandlersMap = Mapping[Union[int, Type[Exception]], ExceptionHandler]
MaybePartial = Union[T, partial]
Middleware = Union[
    Callable[..., ASGIApp], DefineMiddleware, Iterator[Tuple[ASGIApp, Dict[str, Any]]], Type[MiddlewareProtocol]
]
ParametersMap = Mapping[str, ParameterKwarg]
PathType = Union[Path, PathLike, str]
ResponseCookies = Union[Sequence[Cookie], Mapping[str, str]]
ResponseHeaders = Union[Sequence[ResponseHeader], Mapping[str, str]]
Scopes = Set[Literal[ScopeType.HTTP, ScopeType.WEBSOCKET]]
StreamType = Union[Iterable[T], Iterator[T], AsyncIterable[T], AsyncIterator[T]]
TypeEncodersMap = Mapping[Any, Callable[[Any], Any]]



from __future__ import annotations

import uuid
from typing import TYPE_CHECKING, Any

from sqlalchemy.dialects.postgresql import JSONB as PG_JSONB
from sqlalchemy.dialects.postgresql import UUID as PG_UUID
from sqlalchemy.types import CHAR, TypeDecorator
from sqlalchemy.types import JSON as _JSON

if TYPE_CHECKING:
    from sqlalchemy.engine import Dialect


class GUID(TypeDecorator):
    """Platform-independent GUID type.

    Uses PostgreSQL's UUID type, otherwise uses
    CHAR(32), storing as stringified hex values.

    Will accept stringified UUIDs as a hexstring or an actual UUID

    """

    impl = CHAR
    cache_ok = True

    def load_dialect_impl(self, dialect: Dialect) -> Any:
        if dialect.name == "postgresql":
            return dialect.type_descriptor(PG_UUID())
        return dialect.type_descriptor(CHAR(32))

    def process_bind_param(self, value: str | uuid.UUID | None, dialect: Dialect) -> str | None:
        if value is None:
            return value
        if dialect.name == "postgresql":
            return str(value)

        if not isinstance(value, uuid.UUID):
            return "%.32x" % uuid.UUID(value).int

        # hexstring
        return "%.32x" % value.int

    def process_result_value(self, value: str | uuid.UUID | None, dialect: Dialect) -> uuid.UUID | None:
        if value is None:
            return value
        if not isinstance(value, uuid.UUID):
            return uuid.UUID(value)
        return value


class JSON(_JSON):
    """Platform-independent JSON type.

    Uses JSONB type for postgres, otherwise uses the generic JSON data type.
    """

    def load_dialect_impl(self, dialect: Dialect) -> Any:
        if dialect.name == "postgresql":
            return dialect.type_descriptor(PG_JSONB())  # type: ignore[no-untyped-call]
        return dialect.type_descriptor(_JSON())



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any
from urllib.parse import urlencode

__all__ = ("ResponseCacheConfig", "default_cache_key_builder", "CACHE_FOREVER")


if TYPE_CHECKING:
    from litestar import Litestar
    from litestar.connection import Request
    from litestar.stores.base import Store
    from litestar.types import CacheKeyBuilder


class CACHE_FOREVER:  # noqa: N801
    """Sentinel value indicating that a cached response should be stored without an expiration, explicitly skipping the
    default expiration
    """


def default_cache_key_builder(request: Request[Any, Any, Any]) -> str:
    """Given a request object, returns a cache key by combining the path with the sorted query params.

    Args:
        request: request used to generate cache key.

    Returns:
        A combination of url path and query parameters
    """
    query_params: list[tuple[str, Any]] = list(request.query_params.dict().items())
    query_params.sort(key=lambda x: x[0])
    return request.url.path + urlencode(query_params, doseq=True)


@dataclass
class ResponseCacheConfig:
    """Configuration for response caching.

    To enable response caching, pass an instance of this class to :class:`Litestar <.app.Litestar>` using the
    ``response_cache_config`` key.
    """

    default_expiration: int | None = 60
    """Default cache expiration in seconds used when a route handler is configured with ``cache=True``."""
    key_builder: CacheKeyBuilder = field(default=default_cache_key_builder)
    """:class:`CacheKeyBuilder <.types.CacheKeyBuilder>`. Defaults to :func:`default_cache_key_builder`."""
    store: str = "response_cache"
    """Name of the :class:`Store <.stores.base.Store>` to use."""

    def get_store_from_app(self, app: Litestar) -> Store:
        """Get the store defined in :attr:`store` from an :class:`Litestar <.app.Litestar>` instance."""
        return app.stores.get(self.store)



from litestar.template.base import TemplateEngineProtocol, TemplateProtocol
from litestar.template.config import TemplateConfig

__all__ = ("TemplateEngineProtocol", "TemplateProtocol", "TemplateConfig")
